

import com.mindfusion.common.DateTime;
import com.mindfusion.common.Orientation;
import com.mindfusion.scheduling.Calendar;
import com.mindfusion.scheduling.CalendarView;
import com.mindfusion.scheduling.*;
import com.mindfusion.scheduling.model.*;

import java.awt.*;
import java.util.*;
import java.util.List;

import javax.swing.*;

public class Application extends JFrame 
{
	public static final TreeItems DEFAULT_TREE_ITEM = TreeItems.TIMETABLE;
	public static final ThemeType DEFAULT_THEME = ThemeType.Silver;
	
	private static Dimension D_APPLICATION_MIN = new Dimension(900, 650);
	
	private static Dimension D_DESCRIPTION_PREFERRED = new Dimension(D_APPLICATION_MIN.width, (int) (D_APPLICATION_MIN.height * 0.12));
	private static Dimension D_SETTINGS_PREFERRED = new Dimension(D_APPLICATION_MIN.width, (int) (D_APPLICATION_MIN.height * 0.15));
	private static Dimension D_TREE_PREFERRED = new Dimension(
			(int) (D_APPLICATION_MIN.width * 0.25), D_APPLICATION_MIN.height - D_DESCRIPTION_PREFERRED.height - D_SETTINGS_PREFERRED.height); 

	private static String APPLICATION_TITLE = "MindFusion.Scheduling Demo"; 
	
	private static final FileTypeFilter XML_FILTER = new FileTypeFilter(".xml", "MindFusion.Scheduling Files");
	private static final FileTypeFilter JSON_FILTER = new FileTypeFilter(".json", "MindFusion.Scheduling Files");
	
	private static Application instance = null;
	private static List<Image> images = ImageParser.parseImages("resources/pictures");
	
	private Calendar calendar;
	private Description description;   
	private TreeModel tree;
	private SettingsPanel settings;

	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				Application window = null;
				try {
					window = getInstance();
					window.setVisible(true);
				}
				catch (Exception exp) {
				}
			}
		});
	}

	public Application()
	{
		super(APPLICATION_TITLE);
		
		addKeyListener(new KeyboardShortcutsListener());
		
		configureFrame();
		addScheduleData();
		
		System.out.println("currentView : " + calendar.getCurrentView());
		System.out.println("images : " + images.size());
	}
	
	public static Application getInstance()
	{
		if (instance == null)
		{
			instance = new Application();
		}
		
		return instance;
	}
	
	public static List<Image> getImages()
	{
		return images;
	}
	
	public static void setSizeOf(Component component, Dimension size) //to-do : delete it
	{
		component.setSize(size);
		component.setPreferredSize(size);
		component.setMinimumSize(size); 
		component.setMaximumSize(size);
	}
	
	public void updateApplication(TreeItems newItem)
	{
		this.getContentPane().remove(description);
		this.getContentPane().remove(tree);
		this.getContentPane().remove(calendar);
		this.getContentPane().remove(settings);
		
		setView(newItem);
		this.description = new Description(newItem);

		if (this.settings != null)
			this.settings.removeListener(calendar);

		this.settings = new SettingsPanel(calendar, newItem);
		
		configureFields();
		addComponentsIntoFrame();
	}
	
	public void save()
	{
		JFileChooser save = new JFileChooser();
		save.setDialogTitle("Save schedule");
		save.setAcceptAllFileFilterUsed(false);
		save.setFileFilter(JSON_FILTER);
		save.setFileFilter(XML_FILTER);
		
		int retrieval = save.showSaveDialog(null);
		
		if (retrieval == JFileChooser.APPROVE_OPTION) 
		{
			String filePath = save.getSelectedFile().getPath();
			
			if (save.getFileFilter().equals(XML_FILTER))
			{
				filePath += ".xml";
				calendar.getSchedule().saveTo(filePath, ContentType.Xml);
				
			}
			else if (save.getFileFilter().equals(JSON_FILTER))
			{
				filePath += ".json";
				calendar.getSchedule().saveTo(filePath, ContentType.Json);
			}
		}
	}
	
	public void load()
	{
		JFileChooser load = new JFileChooser();
		load.setAcceptAllFileFilterUsed(false);
		
		load.addChoosableFileFilter(XML_FILTER);
		load.addChoosableFileFilter(JSON_FILTER);
		
		int retrieval = load.showOpenDialog(null);
		
		if (retrieval == JFileChooser.APPROVE_OPTION) 
		{
			String filePath = load.getSelectedFile().getPath();
			
			if (load.getFileFilter().equals(XML_FILTER))
			{
				calendar.getSchedule().loadFrom(filePath, ContentType.Xml);
			}
			else if (load.getFileFilter().equals(JSON_FILTER))
			{
				calendar.getSchedule().loadFrom(filePath, ContentType.Json);
			}
		}
		
		calendar.setGroupType(GroupType.None);
		calendar.getContacts().clear();
		calendar.getLocations().clear();
		calendar.getTasks().clear();
		
		calendar.invalidate();
	}
	
	public Calendar getCalendar()
	{
		return this.calendar;
	}
	
	private void configureFrame()
	{	
		this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		this.setExtendedState(JFrame.MAXIMIZED_BOTH);
		this.setMinimumSize(D_APPLICATION_MIN);
		
		this.getContentPane().setLayout(new BorderLayout(5,5));
		this.setJMenuBar(new ApplicationMenu());
		
		configureFields();
		addComponentsIntoFrame();
	}
	
	private void configureFields()
	{
		if (this.calendar == null)
			configureCalendar();
		
		if (this.description == null)
			this.description = new Description(DEFAULT_TREE_ITEM);
		
		if (this.tree == null)
			this.tree = new TreeModel();
		
		if (this.settings == null)
			this.settings = new SettingsPanel(calendar, DEFAULT_TREE_ITEM);
		
		this.description.setPreferredSize(D_DESCRIPTION_PREFERRED);
		this.tree.setPreferredSize(D_TREE_PREFERRED);
		this.settings.setPreferredSize(D_SETTINGS_PREFERRED);
	}
	
	private void configureCalendar()
	{
		this.calendar = new Calendar();
		this.calendar.setTheme(DEFAULT_THEME);
		this.calendar.getSchedule().setUndoEnabled(true);
		this.calendar.addKeyListener(new KeyboardShortcutsListener()); // can catch shortcut events
		
		setView(DEFAULT_TREE_ITEM);

		//for monthSettings
		/*_calendar.BeginInit();
			_calendar.MonthSettings.SubHeaderHeight = 20;
			_calendar.MonthSettings.WeekNumbersHeaderWidth = 20;
			_calendar.EndInit();
		*/
	}
	
	private void addComponentsIntoFrame()
	{
		this.add(description, BorderLayout.PAGE_START);
		this.add(tree, BorderLayout.LINE_START);
		this.add(calendar, BorderLayout.CENTER);
		this.add(settings, BorderLayout.PAGE_END);
		
		this.setPreferredSize(getSize());
		this.pack();
		this.repaint();
	}
	
	private void setView(TreeItems item) // may remove calendar as a argument
	{
		this.calendar.setInteractiveItemType(Appointment.class); //default 
		this.calendar.getItemSettings().setUseStyledText(false); //default 
		
		switch(item)
		{
			case TIMETABLE: 
				timetable(calendar);
				break;
			case VERTICAL_TIMETABLE:
				timetable(calendar);
				calendar.getTimetableSettings().setOrientation(Orientation.Vertical);
				break;
			case HORIZONTAL_TIMETABLE:
				timetable(calendar);
				calendar.getTimetableSettings().setOrientation(Orientation.Horizontal);
				break;
			case TIMETABLE_CUSTOM_DRAWING:
				timetable(calendar);
				break;
			case TIMETABLE_CUSTOM_DRAWING_COLORING_CELLS:
				timetable(calendar);
				break;
			case TIMETABLE_CUSTOM_DRAWING_EMBELISHING:
				timetable(calendar);
				break;
			
			case RESOURCE_VIEW: 
				resourseView(calendar);
				break;
				
			case SINGLE_MONTH: 
				singleMonth(calendar);
				break;
			case SINGLE_MONTH_RESPONDING: 
				singleMonth(calendar);
				singleMonthResponding(calendar);
				break;
			case SINGLE_MONTH_CUSTOMIZING: // to-do fix the problem with "btnChangeButtonStyle" in class SettingsPanel
				singleMonth(calendar);
				break;
			
			case LIST_VIEW: 
				list(calendar);
				break;
			case VERTICAL_LIST_VIEW:
				list(calendar);
				calendar.getListViewSettings().setOrientation(Orientation.Vertical);
				break;
			case HORIZONTAL_LIST_VIEW:
				list(calendar);
				calendar.getListViewSettings().setOrientation(Orientation.Horizontal);
				break;
			
			case WEEK_RANGE: 
				weekRange(calendar);
				break;
			case MULTIPLE_WEEK_RANGE:
				weekRange(calendar);
				calendar.getWeekRangeSettings().setViewStyle(WeekRangeViewStyle.WeekPerRow);
				break;
			case SINGLE_WEEK_RANGE:
				weekRange(calendar);
				calendar.getWeekRangeSettings().setViewStyle(WeekRangeViewStyle.SingleWeek);
				break;
				
			case MONTH_RANGE: 
				monthRange(calendar);
				break;
				
			case MISCELLANEOUS_CUSTOM_ITEM_EDITOR:
				miscellaneous(calendar);
				setItemsVisibility(false);
				break;
				
			case MISCELLANEOUS_FORMATTED_TEXT:
				miscellaneous(calendar);
				setItemsVisibility(true);
				calendar.getItemSettings().setUseStyledText(true);
				break;
		}

		if (item == TreeItems.TIMETABLE)
		{
			calendar.setScrollPosition(
				new Point(calendar.getScrollPosition().x, 12));
		}
	}
	
	private void timetable(Calendar calendar) 
	{
		calendar.setCurrentView(CalendarView.Timetable);

		TimetableSettings settings = calendar.getTimetableSettings();
		settings.getStyle().setHeaderBrush(null);
		settings.getTimelineStyle().setHeaderBrush(null);

		settings.getDates().clear();
		for (int i = 0; i < 5; i++)
		{
			settings.getDates().add(
				DateTime.today().addDays(i));
		}
		settings.setVisibleColumns(4);

		calendar.setDate(DateTime.today());
		calendar.setEndDate(DateTime.today().addDays(60));

		settings.setSecondTimelineOffset(2);
	}

	private void resourseView(Calendar calendar) // to-do: fix the graphic bug with resources
	{
		calendar.setCurrentView(CalendarView.ResourceView);
		calendar.getResourceViewSettings().setVisibleRows(3);
		 
		if (calendar.getGroupType() != GroupType.GroupByContacts &&
			calendar.getGroupType() != GroupType.GroupByTasks &&
			calendar.getGroupType() != GroupType.GroupByLocations &&
			calendar.getGroupType() != GroupType.GroupByResources)
			calendar.setGroupType(GroupType.GroupByContacts);
		
		calendar.addCalendarListener(new CalendarAdapter()
		{
			@Override
			public void dateClick(ResourceDateEvent e)
			{
				if (e.getClicks() == 2 && e.getButton() == java.awt.event.MouseEvent.BUTTON1)
				{
					calendar.resetDrag();
	
					Point mousePoint = MouseInfo.getPointerInfo().getLocation(); 
					Point contactPoint = new Point(
							(int) (mousePoint.getX() - tree.getWidth()),
							(int) (mousePoint.getY() - description.getHeight()));
					
					Contact c = calendar.getContactAt(contactPoint);
					
					if (c == null)
						return;
					
					ResourceForm form = new ResourceForm();
					
					if (!form.isShowing() && form.getResourceName() != null)
					{
						Appointment app = new Appointment();
	
						app.setStartTime(e.getDate());
						app.setEndTime(e.getDate().addDays(2));
						app.setHeaderText(form.getResourceName());
						app.getContacts().add(c);
	
						calendar.getSchedule().getItems().add(app);
					}
				}
			}
		});
	}
	
	private void singleMonth(Calendar calendar)
	{
		 calendar.setCurrentView(CalendarView.SingleMonth);
	}
	
	private void singleMonthResponding(Calendar calendar)
	{
		MonthMenu months = new MonthMenu();
		ItemsMenu menuItems = new ItemsMenu();
		
		calendar.addCalendarListener(new CalendarAdapter()
		{
			@Override
			public void monthHeaderClick(DateEvent e) 
			{
				if (e.getButton() != java.awt.event.MouseEvent.BUTTON1) return; // accept only left mouse click
				
				//open popup menu
				int x = MouseInfo.getPointerInfo().getLocation().x - tree.getWidth();
				int y = e.getBounds().y;
					
				months.show(calendar, x, y); 
			}
			
			@Override
			public void dateClick(ResourceDateEvent e)
			{
				if (e.getButton() != java.awt.event.MouseEvent.BUTTON3) return; // accept only right mouse click
				
				//open popup menu on date
				menuItems.setSelectedDate(e.getDate());
					
				int x = MouseInfo.getPointerInfo().getLocation().x - 233;
				int y = MouseInfo.getPointerInfo().getLocation().y - 146;
					
				menuItems.show(calendar, x, y);
			}
		});
	}
	
	private void list(Calendar calendar) 
	{
		if (calendar.getListViewSettings().getGroupHeaderSize() <= 0)
			calendar.getListViewSettings().setGroupHeaderSize(20);
		calendar.setCurrentView(CalendarView.List);
	}
	
	private  void weekRange(Calendar calendar)
	{
		 calendar.setCurrentView(CalendarView.WeekRange);
	}
	
	private void monthRange(Calendar calendar) 
	{
		 calendar.setCurrentView(CalendarView.MonthRange);
	}
	
	private void miscellaneous(Calendar calendar)
	{
		calendar.setInteractiveItemType(MyAppointment.class); 
		calendar.setCurrentView(CalendarView.Timetable);
		calendar.getTimetableSettings().setOrientation(Orientation.Vertical);
	}
	
	private void setItemsVisibility(boolean visible) // for MISCELLANEOUS_CUSTOM_ITEM_EDITOR and MISCELLANEOUS_FORMATTED_TEXT cases
	{
		Item fe1 = calendar.getSchedule().getItems().get("fe1");
		if (fe1 != null)
		{
			fe1.setVisible(visible);
			if (visible)
				calendar.ensureVisible(fe1);
		}

		Item fe2 = calendar.getSchedule().getItems().get("fe2");
		if (fe2 != null)
		{
			fe2.setVisible(visible);
			if (visible) 
				calendar.ensureVisible(fe2);
		}
	}
	
	private void addScheduleData()
	{
		calendar.beginInit();

		DateList dates = calendar.getTimetableSettings().getDates();
		ResourceList<Contact> contacts = calendar.getSchedule().getContacts();

		Appointment app = null;
		Recurrence recurrence = null;
		DateTime date = DateTime.today();
		Random random = new Random(DateTime.now().getMillisecond());

		// Create some events
		date = dates.get(random.nextInt(dates.size()));
		app = new Appointment();
		app.setStartTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 8, 30, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 10, 30, 0));
		app.setHeaderText("Meet George");
		app.setDescriptionText("This is the second time I promise George. This time I should go.");
		//app.Style = this._eventStyle;
		calendar.getSchedule().getItems().add(app);

		date = dates.get(random.nextInt(dates.size()));
		app = new Appointment();
		app.setStartTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 13, 15, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 14, 00, 0));
		app.setHeaderText("Visit the dentist");
		//app.Style = this._eventStyle;
		calendar.getSchedule().getItems().add(app);

		// Add a recurring appointment
		date = dates.get(0);
		app = new Appointment();
		app.setStartTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 12, 00, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 13, 30, 0));
		app.setHeaderText("Lunch with Laura");
		app.setDescriptionText("Never forget the appointment with the boss.");

		// Each working day, for an eternity
		recurrence = new Recurrence();
		recurrence.setPattern(RecurrencePattern.Daily);
		recurrence.setDailyRecurrence(DailyRecurrence.EveryWorkday);
		recurrence.setStartDate(date);
		recurrence.setRecurrenceEnd(RecurrenceEnd.Never);
		app.setRecurrence(recurrence);

		//app.Style = this._eventStyle;
		calendar.getSchedule().getItems().add(app);

		date = dates.get(random.nextInt(dates.size()));
		app = new Appointment();
		app.setAllDayEvent(true);
		app.setStartTime(date);
		app.setEndTime(date);
		app.setHeaderText("William's birthday");
		//app.Style = this._freeStyle;
		calendar.getSchedule().getItems().add(app);

		date = dates.get(random.nextInt(dates.size()));
		app = new Appointment();
		app.setStartTime( new DateTime(date.getYear(), date.getMonth(), date.getDay(), 10, 00, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 16, 00, 0));
		app.setHeaderText("Have fun!");
		app.setDescriptionText("Some leisure time.");
		//app.Style = this._freeStyle;
		calendar.getSchedule().getItems().add(app);

		date = dates.get(random.nextInt(dates.size()));
		app = new Appointment();
		app.setStartTime( new DateTime(date.getYear(), date.getMonth(), date.getDay(), 18, 20, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 18, 35, 0));
		app.setHeaderText("Prepare the dinner!");
		//app.Style = this._taskStyle;
		calendar.getSchedule().getItems().add(app);

		date = dates.get(random.nextInt(dates.size()));
		app = new Appointment();
		app.setStartTime( new DateTime(date.getYear(), date.getMonth(), date.getDay(), 17, 00, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 17, 45, 0));
		app.setHeaderText("Get the children from the kindergarten!");
		//app.Style = this._taskStyle;
		calendar.getSchedule().getItems().add(app);

		date = dates.get(random.nextInt(dates.size()));
		app = new Appointment();
		app.setStartTime( new DateTime(date.getYear(), date.getMonth(), date.getDay(), 17, 30, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 21, 0, 0));
		app.setHeaderText("Arrange the house payments!");
		//app.Style = this._taskStyle;
		calendar.getSchedule().getItems().add(app);

		date = dates.get(random.nextInt(dates.size()));
		app = new Appointment();
		app.setStartTime( new DateTime(date.getYear(), date.getMonth(), date.getDay(), 7, 30, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 8, 20, 0));
		app.setHeaderText("Drive kids to school!");
		//app.Style = this._taskStyle;
		calendar.getSchedule().getItems().add(app);

		// Add some contacts
		Contact c = null;

		c = new Contact();
		c.setFirstName("Tom");
		c.setLastName("Miles");
		contacts.add(c);
		calendar.getContacts().add(c);

		c = new Contact();
		c.setFirstName("Jimmy");
		c.setLastName("King");
		contacts.add(c);
		calendar.getContacts().add(c);

		c = new Contact();
		c.setFirstName("Elisabeth");
		c.setLastName("Tailor");
		contacts.add(c);
		calendar.getContacts().add(c);

		// Create an item with a styled text...
		date = dates.get(0);
		app = new Appointment();
		app.setStartTime( new DateTime(date.getYear(), date.getMonth(), date.getDay(), 0, 0, 0));
		app.setEndTime(new DateTime(date.getYear(), date.getMonth(), date.getDay(), 5, 0, 0));
		app.setHeaderText("Formatted event");
		app.setDescriptionText("The description of appointments can contain " +
			"<b>bold</b>, <i>italic</i> and <u>underlined</u> texts as well as " +
			"<color=#00ff33>text</color> <color=#ccffcc>in</color> " +
			"<color=#000000>various</color> <color=#ff00cc>colors</color>," +
			"<sup>superscript</sup> and <sub>subscript</sub>.");
		app.setId("fe1");
		app.setVisible(false);
		calendar.getSchedule().getItems().add(app);

		// ...and item with styled text for the first contact
		app = app.clone();
		app.setId("fe2");
		app.getContacts().add(contacts.get(0));
		calendar.getSchedule().getItems().add(app);

		// Add some appointments associated with various contacts
		// so that grouped views are not initially empty
		for (int i = 0; i < 5; i++)
		{
			c = contacts.get(random.nextInt(contacts.size()));
			date = dates.get(0).addDays(random.nextInt(10));
			app = new Appointment();
			app.setStartTime(date);
			app.setEndTime(date.addDays(2.0));
			app.setHeaderText("Event for " + c.getFirstName());
			app.getContacts().add(c);
			calendar.getSchedule().getItems().add(app);
		}
		
		calendar.endInit();
	}
}
