

import java.awt.Dimension;
import java.awt.event.*;
import java.util.HashMap;
import java.util.Map;

import javax.swing.*;

import com.mindfusion.common.DateTime;
import com.mindfusion.scheduling.Calendar;
import com.mindfusion.scheduling.model.Appointment;
import com.mindfusion.scheduling.standardforms.AppointmentForm;
import com.mindfusion.scheduling.standardforms.DialogResult;

public class AppointmentsForm extends Form
{
	private final int SCROLL_BAR_WIDTH = 25; //px
	
	private final String SUBJECT = "Subject"; // to-do: create enum or change values
	private final String START = "From"; 
	private final String END = "To"; 
	
	private AppointmentForm form;
	private Appointment lastSelectedAppointment;
	private Calendar calendar = Application.getInstance().getCalendar();
	
	private Map<Appointment, JPanel> appointmentsPanels;

	public AppointmentsForm(String frameText) 
	{
		super(frameText);
		
		super.getMenu().getBtnAdd().addActionListener(x ->
		{
			if	(form.isEnabled())
			{
				form.dispose();
					
				form = new AppointmentForm(calendar.getSchedule());
				setFormListener();
			}
			form.setVisible(true);
		});	
		
		super.getMenu().getBtnRemove().addActionListener(x -> 
		{
			if	(form.isEnabled())
			{
				form.dispose();
			}
				
			if (lastSelectedAppointment != null)
			{
				calendar.getSchedule().getAllItems().remove(lastSelectedAppointment);
				update();
			}
		});	
		
		addSomeAppointments();
		
		appointmentsPanels = new HashMap<>();
		calendar.getSchedule().getAllItems().forEach(x -> 
		{
			if (x instanceof Appointment)
			{
				JPanel pnlAppointment = rowOfAppointment((Appointment) x);
				appointmentsPanels.put((Appointment) x, pnlAppointment);
			}
		});
		
		form = new AppointmentForm(calendar.getSchedule());
		setFormListener();
		
		makeForm();
		
		super.pack();
		super.repaint();
	}

	@Override
	protected void makeForm() 
	{
		JPanel content = new JPanel();
		content.setLayout(new BoxLayout(content, BoxLayout.Y_AXIS));
		content.setBackground(BACKGROUND);
		
		content.add(rowOfAppointment(null)); // make first row (Subject From To)
		
		if(calendar.getSchedule().getAllItems().isEmpty() && lastSelectedAppointment == null)
		{
			addSomeAppointments();
		}
		
		calendar
		.getSchedule()
		.getAllItems()
		.forEach(x -> 
		{
			if (x instanceof Appointment && !appointmentsPanels.containsKey(x))
			{
				JPanel pnlAppointment = rowOfAppointment((Appointment) x);
				appointmentsPanels.put((Appointment) x, pnlAppointment);
			}
		});
		
		appointmentsPanels.forEach((appointment, panel) ->
		{
			content.add(panel);
		});
		
		JScrollPane pane = new JScrollPane(content);
		pane.setLayout(new ScrollPaneLayout());
		pane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		pane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		
		this.setContentPane(pane);
	}
	
	private JPanel rowOfAppointment(Appointment appointment)
	{
		int rowHeight = (int) (SIZE.getHeight() * 0.05);
		int rowWidth = (int) (SIZE.getWidth() - SCROLL_BAR_WIDTH);
		
		int subjectWidth = rowWidth / 2;
		int timeWidth = rowWidth / 4;
		
		JPanel row = new JPanel();
		row.setLayout(new BoxLayout(row, BoxLayout.X_AXIS));
		row.setBackground(BACKGROUND);
		Application.setSizeOf(row, new Dimension(rowWidth, rowHeight));
		
		JPanel subject = new JPanel();
		subject.setBackground(BACKGROUND);
		Application.setSizeOf(subject, new Dimension(subjectWidth, rowHeight));
		subject.addMouseListener(new MouseAdapter() 
		{
			@Override
			public void mouseClicked(MouseEvent arg0) 
			{
				if (appointment != null)
				{
					lastSelectedAppointment = appointment;
					unselectOtherPannels();
					subject.setBackground(SELECTED_COLOR);
					
					if(arg0.getClickCount() >= 2)
					{
						form.setAppointment(appointment);
						form.setVisible(true);
					}
				}
			}
		});
		
		JPanel startTime = new JPanel();
		startTime.setBackground(BACKGROUND);
		Application.setSizeOf(startTime, new Dimension(timeWidth, rowHeight));
		
		JPanel endTime = new JPanel();
		endTime.setBackground(BACKGROUND);
		Application.setSizeOf(endTime, new Dimension(timeWidth, rowHeight));
		
		if (appointment == null)
		{
			subject.add(new JLabel(SUBJECT));
			startTime.add(new JLabel(START));
			endTime.add(new JLabel(END));
		}
		else
		{
			subject.add(new JLabel(appointment.getSubject()));
			startTime.add(new JLabel(formatTime(appointment.getStartTime())));
			endTime.add(new JLabel(formatTime(appointment.getEndTime())));
		}
		
		row.add(subject);
		row.add(startTime);
		row.add(endTime);
		
		return row;
	}
	
	private String formatTime(DateTime time)
	{
		return String.format("%d.%d.%d", time.getDay(), time.getMonth(), time.getYear());
	}
	
	private void setFormListener()
	{
		form.addWindowListener(new WindowAdapter() 
		{
			@Override
			public void windowClosed(WindowEvent arg0) 
			{
				Appointment newAppointment = form.getAppointment();
				
				if (newAppointment == null)
				{
					return;
				}
				
				if	(form.getDialogResult() == DialogResult.OK)
				{
					calendar.getSchedule().getAllItems().add(newAppointment);
					update();
				}
				else if (form.getDialogResult() == DialogResult.Remove)
				{
					calendar.getSchedule().getAllItems().remove(newAppointment);
					appointmentsPanels.remove(newAppointment);
					update();
				}
			}
		});
	}
	
	private void unselectOtherPannels()
	{
		appointmentsPanels.forEach((appointment, panel) ->
		{
			if (!appointment.equals(lastSelectedAppointment))
			{
				panel.getComponent(0).setBackground(BACKGROUND);
			}
		});
	}
	
	private void addSomeAppointments() // to-do delete it
	{
		Appointment a = new Appointment();
		a.setSubject("Meeting"); 
		a.setStartTime(new DateTime(2021, 2, 3)); // year month day
		a.setEndTime(new DateTime(2021, 2, 4));
		
		calendar.getSchedule().getAllItems().add(a);
		
		Appointment b = new Appointment();
		b.setSubject("Shopping"); 
		b.setStartTime(new DateTime(2021, 2, 3)); // year month day
		b.setEndTime(new DateTime(2021, 2, 3));
		
		calendar.getSchedule().getAllItems().add(b);
	}

	@Override
	protected void update() 
	{
		this.getContentPane().removeAll();
		makeForm();
		
		this.pack();
		this.repaint();
	}
}
