

import java.awt.*;
import java.awt.event.*;
import java.time.*;
import java.time.temporal.TemporalAdjusters;
import java.util.LinkedHashMap;
import java.util.Map;

import javax.swing.*;

import com.mindfusion.common.DateTime;
import com.mindfusion.scheduling.DateList;

public class CalendarView extends JFrame 
{
	private static int column = 7;
	private static int MAX_VISIBLE_CALENDAR_CULUMNS = 4;
	
	private static final Dimension D_NAVIGATION_BUTTONS = new Dimension(20, 20);
	private static final Dimension D_BUTTONS = new Dimension(80, 20);
	
	private static final Dimension D_PNL_DAY = new Dimension(30, 30);
	private static final Dimension D_PNL_DAY_OF_WEEK = new Dimension((int) D_PNL_DAY.getWidth(), 20);
	private static final Dimension D_PNL_CALENDAR = new Dimension((int) (++column * D_PNL_DAY.getWidth()) + 26, (int) (++column * D_PNL_DAY.getHeight()));
	
	private static final Dimension D_CALENDAR_VIEW = new Dimension((int) D_PNL_CALENDAR.getWidth(),
			(int) (D_PNL_CALENDAR.getHeight() + D_NAVIGATION_BUTTONS.getHeight() + D_BUTTONS.getHeight() + 50));
	
	private static final Color SELECTED = Color.GRAY;
	private static final Color UNSELECTED = Color.LIGHT_GRAY;
	private static final Color TODAY = Color.ORANGE;
	
	private Map<LocalDate, JPanel> days;
	private LocalDate today = LocalDateTime.now().toLocalDate();
	private LocalDate currentDate;
	
	private LocalDate startDate;
	private LocalDate endDate;
	
	private boolean isPressed = false;
	
	public CalendarView()
	{
		this(LocalDateTime.now().toLocalDate());
	}
	
	public CalendarView(LocalDate time)
	{
		super("Date range");
		configurateCalendarView();

		this.currentDate = time;
		
		int startColumn = currentDate.with(TemporalAdjusters.firstDayOfMonth()).getDayOfWeek().getValue();
		int daysOfMonth = currentDate.with(TemporalAdjusters.lastDayOfMonth()).getDayOfMonth();
		
		makeCalendarView(startColumn, daysOfMonth);
	}
	
	private void configurateCalendarView() // to-do: change values
	{
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE); 
		this.setLayout(new BoxLayout(this.getContentPane(), BoxLayout.Y_AXIS));
		this.setVisible(false);
		this.setLocation((int) 300,//((Application.FRAME_WIDTH - D_CALENDAR_VIEW.getWidth()) / 2), // ?
						 (int) 400);//((Application.FRAME_HEIGHT - D_CALENDAR_VIEW.getHeight()) / 2)); //?
		
		Application.setSizeOf(this, D_CALENDAR_VIEW);
		
		this.days = new LinkedHashMap<>();
	}

	private void makeCalendarView(int startColumn, int daysOfMonth)
	{
		this.add(makeTopPanel());
		
		int year = currentDate.getYear();
		int month = currentDate.getMonthValue();
		
		JPanel calendar = new JPanel();
		calendar.setLayout(new FlowLayout(FlowLayout.LEFT));
		Application.setSizeOf(calendar, D_PNL_CALENDAR);
		
		//make day of week labels
		String[] dayOfWeekLabels = {"M", "T", "W", "T", "F", "S", "S"};
		for (String s : dayOfWeekLabels)
		{
			calendar.add(makeDayOfWeekPanel(s));
		}
		
		//make empty days
		for (int i = 1; i < startColumn ; i++)
		{
			calendar.add(makePanelOfDay(0));
		}
		
		//make calendar days
		for (int i = 1; i <= daysOfMonth; i++) 
		{
			LocalDate key = LocalDate.of(year, month, i);
			
			days.put(key, makePanelOfDay(i));
			
			calendar.add(days.get(key));
		}
		
		this.add(calendar);
		
		this.add(makeBottomPanel());
		
		this.pack();
		this.repaint();
	}
	
	private void repaintCalendarView()
	{
		this.getContentPane().removeAll();
		
		int startColumn = currentDate.with(TemporalAdjusters.firstDayOfMonth()).getDayOfWeek().getValue();
		int daysOfMonth = currentDate.with(TemporalAdjusters.lastDayOfMonth()).getDayOfMonth();
		
		makeCalendarView(startColumn, daysOfMonth);
	}
	
	private JPanel makePanelOfDay(int day)
	{
		JPanel pnlDay = new JPanel();
		
		Application.setSizeOf(pnlDay, D_PNL_DAY);
		
		if (day == 0)
		{
			return pnlDay;
		}
		
		LocalDate date = LocalDate.of(currentDate.getYear(), currentDate.getMonth(), day);
		
		pnlDay.add(new JLabel(String.valueOf(day)));
		pnlDay.setBackground(today.equals(date) ? TODAY : UNSELECTED);
		pnlDay.addMouseListener(new MouseAdapter() 
		{
			@Override
			public void mouseEntered(MouseEvent arg0)
			{	
				if (isPressed)
				{
					pnlDay.setBackground(SELECTED);
					endDate = date;
					allDaysInRange(true);
				}
			}

			@Override
			public void mouseExited(MouseEvent arg0) 
			{	
				if (isPressed)
				{
					pnlDay.setBackground(UNSELECTED);
					if (endDate != null)
					{
						allDaysInRange(false);
						days.get(startDate).setBackground(SELECTED);
					}
				}
			}

			@Override
			public void mousePressed(MouseEvent arg0) 
			{
				//select with shift
				if (arg0.isShiftDown() && startDate != null && !startDate.equals(date))
				{
					endDate = date;
					allDaysInRange(true);
					return;
				}
				
				//select with mouse press
				if (startDate != null && endDate == null && !startDate.equals(date))
				{
					days.get(startDate).setBackground(UNSELECTED);
				}
				
				if (startDate != null && endDate != null)
				{
					allDaysInRange(false);
					endDate = null;
				}
				
				isPressed = true;
				pnlDay.setBackground(SELECTED);
				startDate = date;
			}

			@Override
			public void mouseReleased(MouseEvent arg0)
			{
				isPressed = false;
				
				if (endDate != null && endDate.isBefore(startDate))
				{
					LocalDate temp = endDate;
					endDate = startDate;
					startDate = temp;
				}
			}
		});
	
		return pnlDay;
	}
	
	private void allDaysInRange(boolean areSelected)
	{
		int differense = Math.abs(endDate.getDayOfMonth() - startDate.getDayOfMonth());
		
		for (int i = 0; i <= differense; i++)
		{
			if (endDate.isBefore(startDate))
			{
				days.get(endDate.plusDays(i)).setBackground(areSelected ? SELECTED : UNSELECTED);
			}
			else
			{
				days.get(startDate.plusDays(i)).setBackground(areSelected ? SELECTED : UNSELECTED);
			}
		}
		
		if (!areSelected)	
		{
			days.get(today).setBackground(TODAY);
		}
		
	}
	
	private JPanel makeTopPanel()
	{
		JPanel pnlTop = new JPanel(); 
		pnlTop.setLayout(new BoxLayout(pnlTop, BoxLayout.X_AXIS));
		
		JButton btnPrevious = new JButton(new CircleIcon(true)); // if the triangle is turned to the left -> true
		Application.setSizeOf(btnPrevious, D_NAVIGATION_BUTTONS);
		btnPrevious.addActionListener(x -> 
		{
			goToPreviousMonth();
			repaintCalendarView();
		});
		
		JButton btnNext = new JButton(new CircleIcon(false)); // if the triangle is turned to the right -> false
		Application.setSizeOf(btnNext, D_NAVIGATION_BUTTONS);
		btnNext.addActionListener(x -> 
		{
			goToNextMonth();
			repaintCalendarView();
		});
		
		JLabel lblMonth = new JLabel(String.format(" %s, %d ", currentDate.getMonth().toString(), currentDate.getYear()));
		
		pnlTop.add(btnPrevious);
		pnlTop.add(lblMonth);
		pnlTop.add(btnNext);
		
		return pnlTop;
	}
	
	private JPanel makeDayOfWeekPanel(String s)
	{
		JPanel pnlDay = new JPanel();
		
		Application.setSizeOf(pnlDay, D_PNL_DAY_OF_WEEK);
		
		pnlDay.add(new JLabel(s));
		
		return pnlDay;
	}
	
	private JPanel makeBottomPanel() 
	{
		JPanel pnlBottom = new JPanel();
		pnlBottom.setLayout(new BoxLayout(pnlBottom, BoxLayout.X_AXIS));
		
		JButton btnOk = new JButton("OK");
		Application.setSizeOf(btnOk, D_BUTTONS);
		btnOk.addActionListener(x -> 
		{
			if (startDate == null)
			{
				return;
			}
			
			DateTime start = new DateTime(startDate.getYear(), startDate.getMonth().getValue(), startDate.getDayOfMonth());
			DateTime end = start.addDays(1);
			
			if (endDate != null)
			{
				end = new DateTime(endDate.getYear(), endDate.getMonth().getValue(), endDate.getDayOfMonth());
			}
			
			DateList dates = Application.getInstance().getCalendar().getTimetableSettings().getDates();
			dates.clear();
			
			for (int i = 0; i <= end.getDay() - start.getDay(); i++)
			{
				DateTime date = start.addDays(i);
				dates.add(date);
			}
			
			int visibleColumns = dates.size() > MAX_VISIBLE_CALENDAR_CULUMNS 
					? MAX_VISIBLE_CALENDAR_CULUMNS 
					: dates.size(); 
			
			Application.getInstance().getCalendar().getTimetableSettings().setVisibleColumns(visibleColumns);
			Application.getInstance().getCalendar().getSelection().set(start, end);
			Application.getInstance().getCalendar().setDate(start);
			Application.getInstance().getCalendar().setEndDate(end);
			
			dispose();
		});
		
		JButton btnCancel = new JButton("CANCEL");
		Application.setSizeOf(btnCancel, D_BUTTONS);
		btnCancel.addActionListener(x ->
		{
			dispose();
		});
		
		pnlBottom.add(btnOk);
		pnlBottom.add(btnCancel);
		
		return pnlBottom;
	}
	
	private void goToPreviousMonth()
	{	
		LocalDate previousDate = currentDate.with(TemporalAdjusters.firstDayOfMonth()).minusMonths(1);
		this.currentDate = previousDate;
	}
	
	private void goToNextMonth()
	{
		LocalDate nextDate = currentDate.with(TemporalAdjusters.firstDayOfMonth()).plusMonths(1);
		this.currentDate = nextDate;
	}
	
	private class CircleIcon implements Icon 
	{
		private int[] xValues;
	    private int[] yValues;
	    
		private CircleIcon(boolean isLeft) // to-do : get rid of magical numbers
		{
			if (isLeft)
			{
				xValues = new int[]{7, 12, 12};
				yValues = new int[]{10, 5, 15};
			}
			else 
			{
				xValues = new int[]{13, 8, 8};
				yValues = new int[]{10, 5, 15};
			}
		}
	
		@Override
		public void paintIcon(Component c, Graphics g, int x, int y) 
		{
		    Graphics2D g2 = (Graphics2D) g.create();
		    g2.setColor(Color.black);
		    
		    Polygon triangle = new Polygon(xValues, yValues, 3);
		    
		    g2.draw(triangle);
		    g2.fill(triangle);
		    
		    g2.dispose();
		}

		@Override
		public int getIconWidth() 
		{
		    return (int) D_NAVIGATION_BUTTONS.getWidth();
		}

		@Override
		public int getIconHeight() 
		{
		    return (int) D_NAVIGATION_BUTTONS.getHeight();
		}
	}
}
