

import java.awt.Image;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URISyntaxException;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;
import java.util.jar.JarEntry;
import java.util.jar.JarFile;
import java.util.stream.Stream;

import javax.imageio.ImageIO;

public class ImageParser 
{
	public static List<Image> parseImages(String directory)
	{
		List<Image> images = new ArrayList<>();
		
		if (directory == null)
			throw new NullPointerException("path can't be null");

		try {
			final File jarFile = new File(
					ImageParser.class.getProtectionDomain().getCodeSource().getLocation().getPath());

			if (jarFile.isFile()) {  // Run with JAR file
				final JarFile jar = new JarFile(jarFile);
				final Enumeration<JarEntry> entries = jar.entries(); //gives ALL entries in jar
				while (entries.hasMoreElements()) {
					final String name = entries.nextElement().getName();
					if (name.startsWith(directory + "/")) { //filter according to the path
						images.add(createImage(ImageParser.class.getResourceAsStream(name)));
					}
				}
				jar.close();
			} else { // Run with IDE
				final URL url = ImageParser.class.getResource("/" + directory);
				if (url != null) {
					try {
						final File apps = new File(url.toURI());
						for (File app : apps.listFiles()) {
							images.add(createImage(app));
						}
					} catch (URISyntaxException ex) {
						// never happens
					}
				}
			}
		}
		catch (IOException ex)
		{
		}

		return images;
	}
	
	private static BufferedImage createImage(InputStream stream)
	{
		try 
		{
			BufferedImage image = ImageIO.read(stream);
			return image;
		} 
		catch (IOException e) 
		{
			e.printStackTrace();
		}
		
		return null;
	}

	private static BufferedImage createImage(File file)
	{
		try
		{
			BufferedImage image = ImageIO.read(file);
			return image;
		}
		catch (IOException e)
		{
			e.printStackTrace();
		}

		return null;
	}
}
