

import java.awt.Image;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.IOException;

import javax.imageio.ImageIO;
import javax.swing.*;
import com.mindfusion.common.DateTime;
import com.mindfusion.drawing.ImageAlignment;
import com.mindfusion.scheduling.*;
import com.mindfusion.scheduling.model.Style;

public class ItemsMenu extends JPopupMenu 
{
	private DateTime date;
	
	private Style holidayStyle;
	private Style birthdayStyle;
	
	private Image holidayImage = Application.getImages().get(34);//image40.png
	private Image birthdayImage = Application.getImages().get(3);//image12.png
	
	private Calendar calendar = Application.getInstance().getCalendar();
	
	public ItemsMenu()
	{
		holidayStyle = createItemStyle(holidayStyle, holidayImage); 
		birthdayStyle = createItemStyle(birthdayStyle, birthdayImage); 
		
		Items[] items = Items.class.getEnumConstants();
		
		for	(int i = 0; i < items.length; i++)
		{
			Items item = items[i];
			JMenuItem current = item.getItem();
			current.addActionListener(new ActionListener() 
			{
				@Override
				public void actionPerformed(ActionEvent arg0) 
				{
					switch(item)
					{
						case BIRTHDAY: 
							birthday();
							break;
						case HOLIDAY:
							holiday();
							break;
					}
				}
			});
			
			add(current);
		}
	}
	
	private void holiday()
	{
		// Make the clicked date a holiday if it is not and
		// not a holiday otherwise
		
		boolean wasHoliday = false;
		
		DateStyleList calendarDateStyles = Application.getInstance().getCalendar().getDayStyles();
		
		for (DateStyle style : calendarDateStyles)
		{
			if (style.getFrom().equals(date))
			{
				if (style.getStyle().getImage().equals(holidayImage))
					wasHoliday = true;

				calendar.getDayStyles().remove(style);

				break;
			}
		}

		if (!wasHoliday)
		{
			DateStyle style = new DateStyle();
			style.setFrom(date);
			style.setTo(date);
			style.setStyle(holidayStyle); 
			
			calendar.getDayStyles().add(style);
		}
		
		calendar.invalidate();
	}
	
	private void birthday()
	{
		// Make the clicked date a birthday if it is not and
		// not a birthday otherwise
		
		boolean wasBirthday = false;
		
		DateStyleList calendarDateStyles = Application.getInstance().getCalendar().getDayStyles();
		
		for (DateStyle style : calendarDateStyles)
		{
			if (style.getFrom().equals(date))
			{
				if (style.getStyle().getImage().equals(birthdayImage))
					wasBirthday = true;

				calendar.getDayStyles().remove(style);

				break;
			}
		}

		if (!wasBirthday)
		{
			DateStyle style = new DateStyle();
			style.setFrom(date);
			style.setTo(date);
			style.setStyle(birthdayStyle); 
			
			calendar.getDayStyles().add(style);
		}
		
		calendar.invalidate();
	}
	
	private Style createItemStyle(Style item, Image image)
	{
		item = new Style();
		item.setBorderTopWidth(1);
		item.setBorderLeftWidth(1);
		item.setBorderBottomWidth(1);
		item.setBorderRightWidth(1);
		item.setImage(image);  
		item.setImageAlignment(ImageAlignment.Left);
		
		return item;
	}
	
	public void setSelectedDate(DateTime date)
	{
		if (date != null)
		{
			this.date = date;
		}
	}

}
