import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.mindfusion.scheduling.ItemEditContext;
import com.mindfusion.scheduling.ItemEditor;


public class MyEditor
	extends JPanel
	implements ItemEditor, KeyListener
{
	private ItemEditContext context;
	private JTextField headerField;
	private JTextField descriptionField;
	private JButton okBtn;
	private JButton cancelBtn;

	public MyEditor()
	{
		super(new BorderLayout());

		setVisible(false);
		setBackground(Color.green);

		headerField = new JTextField();
		headerField.addKeyListener(this);

		descriptionField = new JTextField();
		descriptionField.addKeyListener(this);

		okBtn = new JButton("OK");
		okBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				endEdit(context, true);
			}
		});

		cancelBtn = new JButton("X");
		cancelBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				endEdit(context, false);
			}
		});

		JPanel buttonStack = new JPanel();
		buttonStack.setLayout(
			new BoxLayout(buttonStack, BoxLayout.Y_AXIS));
		buttonStack.add(okBtn);
		buttonStack.add(cancelBtn);

		add(headerField, BorderLayout.PAGE_START);
		add(descriptionField, BorderLayout.CENTER);
		add(buttonStack, BorderLayout.LINE_END);
	}

	@Override
	public void startEdit(ItemEditContext context)
	{
		this.context = context;

		String headerText = context.getItem().getHeaderText();
		headerField.setText(headerText);
		headerField.select(
			headerText.length(), headerText.length());
		headerField.requestFocusInWindow();

		descriptionField.setText(
			context.getItem().getDescriptionText());
	}

	@Override
	public void endEdit(ItemEditContext context, boolean accept)
	{
		if (accept)
		{
			context.getItem().setHeaderText(headerField.getText());
			context.getItem().setDescriptionText(descriptionField.getText());

			context.accept();
		}
		else
		{
			context.cancel();
		}
	}

	@Override
	public String getText()
	{
		return headerField.getText();
	}

	private static final long serialVersionUID = 1L;

	@Override public void keyTyped(KeyEvent e) {}
	@Override public void keyPressed(KeyEvent e) {}

	@Override
	public void keyReleased(KeyEvent e)
	{
		if (e.getKeyCode() == KeyEvent.VK_ESCAPE)
			endEdit(context, false);
		if (e.getKeyCode() == KeyEvent.VK_ENTER)
			endEdit(context, true);
	}
}
