

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.image.BufferedImage;
import java.util.EnumSet;

import javax.swing.*;

import com.mindfusion.scheduling.model.ShadowStyle;
import com.mindfusion.common.DateTime;
import com.mindfusion.common.DayOfWeek;
import com.mindfusion.common.Duration;
import com.mindfusion.common.Orientation;
import com.mindfusion.scheduling.*;
import com.mindfusion.drawing.*;

public class SettingsPanel extends JPanel
{	
	private static final Insets INSETS = new Insets(10, 10, 10, 10); 
	
	private TreeItems currentItem;
	private CalendarListener listener;
	
	private int embelishTimeline; //0 - no; 1 - emb.1; 2 - emb.2
	private boolean embelishMonthHeader;
	private DateStyle style1;
	private DateStyle style2;
	private DateStyle style3;
	private DateStyle style4;
	
	public SettingsPanel(Calendar calendar, TreeItems item)
	{
		super();
		
		currentItem = item;
		
		this.setLayout(new FlowLayout(FlowLayout.LEADING));
		this.setBorder(BorderFactory.createLineBorder(Color.black));
		
		addListener(calendar);
		
		switch(item)
		{
			case TIMETABLE: 
				this.add(timeTableSettings(calendar)); 
				break;
				
			case TIMETABLE_CUSTOM_DRAWING:
				this.add(customDrawing(calendar));
				break;
				
			case TIMETABLE_CUSTOM_DRAWING_COLORING_CELLS:
				this.add(customDrawingColoringCells(calendar));
				break;
				
			case TIMETABLE_CUSTOM_DRAWING_EMBELISHING:
				this.add(customDrawingEmbelishingSettings(calendar));
				break;
				
			case RESOURCE_VIEW: 
				this.add(resourceViewSettings(calendar)); 
				break;
				
			case SINGLE_MONTH: 
				this.add(singleMonthSettings(calendar)); 
				break;
				
			case SINGLE_MONTH_CUSTOMIZING:
				this.add(singleMonthCustomizingSettings(calendar));
				break;
			
			case LIST_VIEW: 
				this.add(listViewSettings(calendar)); 
				break;
			
			case WEEK_RANGE: 
				this.add(weekRangeSettings(calendar)); 
				break;
				
			case MONTH_RANGE: 
				this.add(monthRangeSettings(calendar));
				break;
		}
	}
	
	private JPanel customDrawing(Calendar calendar) 
	{
		calendar.setCustomDraw(
			EnumSet.of(
				CustomDrawElements.TimetableCell,
				CustomDrawElements.TimetableWholeDayHeader,
				CustomDrawElements.TimetableColumnHeader));

		return new JPanel();
	}

	private JPanel customDrawingColoringCells(Calendar calendar)
	{
		calendar.setCustomDraw(
			EnumSet.of(CustomDrawElements.TimetableCell));
		return new JPanel();
	}

	private JPanel customDrawingEmbelishingSettings(Calendar calendar) 
	{
		EnumSet<CustomDrawElements> values = EnumSet.of(CustomDrawElements.TimetableTimelineHourCell, 
				CustomDrawElements.TimetableTimelineCell);
		calendar.setCustomDraw(values);
		
		JButton btnAddEmbelishment1 = new JButton("Add embelishment 1");
		btnAddEmbelishment1.addActionListener(x ->
		{
			embelishTimeline |= 1;
			calendar.invalidate();
		});
		
		JButton btnAddEmbelishment2 = new JButton("Add embelishment 2");
		btnAddEmbelishment2.addActionListener(x -> 
		{
			embelishTimeline |= 2;
			calendar.invalidate();
		});
		
		JButton btnClear = new JButton("clear all embelishments");
		btnClear.addActionListener(x -> 
		{
			embelishTimeline = 0;
			calendar.invalidate();
		});
		
		//Put the components in the panel
		JPanel pnlCustomDrawingEmbelishingSettings = new JPanel();
		pnlCustomDrawingEmbelishingSettings.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = INSETS;
		
		pnlCustomDrawingEmbelishingSettings.add(btnAddEmbelishment1, c);
		
		c.gridx = 2;
		pnlCustomDrawingEmbelishingSettings.add(btnAddEmbelishment2, c);
		
		c.gridx = 3;
		pnlCustomDrawingEmbelishingSettings.add(btnClear, c);
		
		return pnlCustomDrawingEmbelishingSettings;
	}

	private JPanel monthRangeSettings(Calendar calendar) 
	{
		MonthRangeSettings s = calendar.getMonthRangeSettings();
		
		JCheckBox yearNavigationBox = new JCheckBox();
		boolean hasButtons = s.getHeaderStyle().contains(MonthRangeHeaderStyles.Buttons);
		yearNavigationBox.setSelected(hasButtons);
		yearNavigationBox.addActionListener(x ->
		{
			EnumSet<MonthRangeHeaderStyles> components = EnumSet.copyOf(s.getHeaderStyle());
			
			if (yearNavigationBox.isSelected())
			{
				components.add(MonthRangeHeaderStyles.Buttons);
			}
			else
			{
				components.remove(MonthRangeHeaderStyles.Buttons);
			}
			s.setHeaderStyle(components);
		});
		
		JSlider numberOfMonthsSlider = new JSlider();
		numberOfMonthsSlider.setValue(s.getNumberOfMonths());
		numberOfMonthsSlider.addChangeListener(x -> 
		{
			s.setNumberOfMonths(numberOfMonthsSlider.getValue() > 0 ? numberOfMonthsSlider.getValue() : 1); 
		});
		
		JSlider monthsPerRowSlider = new JSlider();
		monthsPerRowSlider.setValue(s.getMonthsPerRow());
		monthsPerRowSlider.addChangeListener(x ->
		{
			if (1 <= monthsPerRowSlider.getValue() && monthsPerRowSlider.getValue() <= 12)
				s.setMonthsPerRow(monthsPerRowSlider.getValue()); 
			else if (monthsPerRowSlider.getValue() > 12)
				s.setMonthsPerRow(12); 
			else
				s.setMonthsPerRow(1); 
		});
		
		JSlider yearRowSlider = new JSlider();
		yearRowSlider.setValue(s.getVisibleRows());
		yearRowSlider.addChangeListener(x -> 
		{
			s.setVisibleRows(yearRowSlider.getValue() > 0 ? yearRowSlider.getValue() : 1);
		});
		
		//Put the components in the panel
		JPanel pnlMonthRangeSettings = new JPanel();
		
		pnlMonthRangeSettings.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = INSETS;
		
		JPanel pnlFirstColumn = (makePanelColumn(
				makePanelRow(yearNavigationBox, new JLabel("Navigation buttons"))
				));
		pnlMonthRangeSettings.add(pnlFirstColumn, c);
		
		c.gridx = 1;
		JPanel pnlSecondColumn = (makePanelColumn(
				makePanelRow(new JLabel("Number of months : "), numberOfMonthsSlider),
				makePanelRow(new JLabel("Months per row : "), monthsPerRowSlider),
				makePanelRow(new JLabel("Rows : "), yearRowSlider)
				));
		pnlMonthRangeSettings.add(pnlSecondColumn, c);
		
		return pnlMonthRangeSettings;
	}
	
	private JPanel weekRangeSettings(Calendar calendar)
	{
		WeekRangeSettings s = calendar.getWeekRangeSettings();
		
		JCheckBox timePeriodHeaderBox = new JCheckBox();
		boolean hasTimePeriod = s.getHeaderStyle().contains(WeekRangeHeaderStyle.Title);
		timePeriodHeaderBox.setSelected(hasTimePeriod);
		timePeriodHeaderBox.addActionListener(x -> 
		{
			EnumSet<WeekRangeHeaderStyle> components = EnumSet.copyOf(s.getHeaderStyle());
			if (timePeriodHeaderBox.isSelected())
			{
				components.add(WeekRangeHeaderStyle.Title);
			}
			else
			{
				components.remove(WeekRangeHeaderStyle.Title);
			}
			s.setHeaderStyle(components);
		});
		
		JCheckBox dayOfWeekHeader2Box = new JCheckBox();
		dayOfWeekHeader2Box.setSelected(s.getShowDayOfWeek());
		dayOfWeekHeader2Box.addActionListener(x -> 
		{
			s.setShowDayOfWeek(dayOfWeekHeader2Box.isSelected());
		});
		
		JCheckBox calendarHeaderShadowBox = new JCheckBox();
		calendarHeaderShadowBox.setSelected(s.getHeaderShadowStyle() == ShadowStyle.Fading);
		calendarHeaderShadowBox.addActionListener(x -> 
		{
			s.setHeaderShadowStyle(calendarHeaderShadowBox.isSelected() ? ShadowStyle.Fading : ShadowStyle.None);
		});
		
		JCheckBox groupWeekendsBox = new JCheckBox();
		groupWeekendsBox.setSelected(s.getGroupWeekends());
		groupWeekendsBox.addActionListener(x -> 
		{
			s.setGroupWeekends(groupWeekendsBox.isSelected());
		});
		
		JSlider calendarHeaderSizeSlider = new JSlider();
		calendarHeaderSizeSlider.setValue(s.getTimePeriodHeaderHeight());
		calendarHeaderSizeSlider.addChangeListener(x -> 
		{
			s.setTimePeriodHeaderHeight(calendarHeaderSizeSlider.getValue());
		});
		
		JSlider subHeaderSizeSlider = new JSlider();
		subHeaderSizeSlider.setValue(s.getDayNamesHeaderHeight());
		subHeaderSizeSlider.addChangeListener(x ->
		{
			s.setDayNamesHeaderHeight(subHeaderSizeSlider.getValue());
		});
		
		JSlider totalDaysSlider = new JSlider(); 
		totalDaysSlider.addChangeListener(x -> 
		{
			double days = Duration.fromDays(totalDaysSlider.getValue()).getTotalDays();
			DateTime value = calendar.getDate().addDays(days);
			calendar.setEndDate(value);
		});
		
		JSlider visibleRowsSlider = new JSlider();
		visibleRowsSlider.setMinimum(1);
		visibleRowsSlider.setMaximum(10);
		visibleRowsSlider.setValue(s.getVisibleRows());
		visibleRowsSlider.addChangeListener(x -> 
		{
			s.setVisibleRows(visibleRowsSlider.getValue()); 
		});
		
		//Put the components in the panel
		JPanel pnlWeekRangeSettings = new JPanel();
		
		pnlWeekRangeSettings.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = INSETS;
		
		JPanel pnlFirstColumn = (makePanelColumn(
				makePanelRow(timePeriodHeaderBox, new JLabel("Time period header")),
				makePanelRow(dayOfWeekHeader2Box, new JLabel("Day of week header")),
				makePanelRow(calendarHeaderShadowBox, new JLabel("Header shadow"))
				));
		pnlWeekRangeSettings.add(pnlFirstColumn, c);
		
		c.gridx = 1;
		JPanel pnlSecondColumn = (makePanelColumn(
				makePanelRow(new JLabel("Header size : "), calendarHeaderSizeSlider),
				makePanelRow(new JLabel("Sub-header size : "), subHeaderSizeSlider)
				));
		pnlWeekRangeSettings.add(pnlSecondColumn, c);
		
		c.gridx = 2;
		JPanel pnlThirdColumn = (makePanelColumn(
				makePanelRow(new JLabel("Total days : "), totalDaysSlider),
				makePanelRow(new JLabel("Visible rows : "), visibleRowsSlider),
				makePanelRow(groupWeekendsBox, new JLabel("Group weekends"))
				));
		pnlWeekRangeSettings.add(pnlThirdColumn, c);
		
		return pnlWeekRangeSettings;
	}
	
	private JPanel listViewSettings(Calendar calendar) 
	{
		ListViewSettings s = calendar.getListViewSettings();
		
		JCheckBox weekHeaderShadowBox = new JCheckBox();
		weekHeaderShadowBox.setSelected(s.getHeaderShadowStyle() == ShadowStyle.Fading);
		weekHeaderShadowBox.addActionListener(x -> 
		{
			s.setHeaderShadowStyle(weekHeaderShadowBox.isSelected() ? ShadowStyle.Fading : ShadowStyle.None);
		});
		
		JSlider weekHeaderSizeSlider = new JSlider();
		weekHeaderSizeSlider.setValue(s.getHeaderSize());
		weekHeaderSizeSlider.addChangeListener(x -> 
		{
			s.setHeaderSize(weekHeaderSizeSlider.getValue());
		});
		
		JSlider visibleDaysSlider = new JSlider();
		JSlider numberOfDaysSlider = new JSlider();
		
		visibleDaysSlider.setValue(s.getVisibleCells());
		visibleDaysSlider.addChangeListener(x -> 
		{
			if (visibleDaysSlider.getValue() > s.getNumberOfCells())
			{
				s.setNumberOfCells(visibleDaysSlider.getValue()); 
				numberOfDaysSlider.setValue(visibleDaysSlider.getValue()); 
			}
			s.setVisibleCells(visibleDaysSlider.getValue() > 0 ? visibleDaysSlider.getValue() : 1);
		});
		
		numberOfDaysSlider.setValue(s.getNumberOfCells());
		numberOfDaysSlider.addChangeListener(x -> 
		{
			if (numberOfDaysSlider.getValue() < s.getVisibleCells())
			{
				s.setVisibleCells(numberOfDaysSlider.getValue() > 0 ? numberOfDaysSlider.getValue() : 1);
				visibleDaysSlider.setValue(numberOfDaysSlider.getValue());
			}
			s.setNumberOfCells(numberOfDaysSlider.getValue() > 0 ? numberOfDaysSlider.getValue() : 1);
		});
		
		//Put the components in the panel
		JPanel pnlListViewSettings = new JPanel();
		
		pnlListViewSettings.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = INSETS;
		
		JPanel pnlFirstColumn = (makePanelColumn(
				makePanelRow(weekHeaderShadowBox, new JLabel("Header shadow"))));
		pnlListViewSettings.add(pnlFirstColumn, c);
		
		c.gridx = 1;
		JPanel pnlSecondColumn = (makePanelColumn(
				makePanelRow(new JLabel("Header size : "), weekHeaderSizeSlider),
				makePanelRow(new JLabel("Total days : "), numberOfDaysSlider),
				makePanelRow(new JLabel("Visible days : "), visibleDaysSlider)
				));
		pnlListViewSettings.add(pnlSecondColumn, c);
		
		return pnlListViewSettings;
	}
	
	private JPanel timeTableSettings(Calendar calendar) 
	{
		TimetableSettings s = calendar.getTimetableSettings();
		
		JSlider timetableHeaderSizeSlider = new JSlider();
		timetableHeaderSizeSlider.setValue(s.getMainHeaderSize());
		timetableHeaderSizeSlider.addChangeListener(x -> 
		{
			s.setMainHeaderSize(timetableHeaderSizeSlider.getValue());
		});
		
		JSlider timetableSubHeaderSizeSlider = new JSlider();
		timetableSubHeaderSizeSlider.setValue(s.getSubHeaderSize());
		timetableSubHeaderSizeSlider.addChangeListener(x -> 
		{
			s.setSubHeaderSize(timetableSubHeaderSizeSlider.getValue());
		});
		
		JCheckBox timetableHeaderShadowBox = new JCheckBox();
		timetableHeaderShadowBox.setSelected(s.getHeaderShadowStyle() == ShadowStyle.Fading);
		timetableHeaderShadowBox.addActionListener(x ->
		{
			s.setHeaderShadowStyle(timetableHeaderShadowBox.isSelected() ? ShadowStyle.Fading : ShadowStyle.None);
		});
		
		JSlider timelineSizeSlider = new JSlider();
		timelineSizeSlider.setValue(s.getTimelineSize());
		timelineSizeSlider.addChangeListener(x -> 
		{
			s.setTimelineSize(timelineSizeSlider.getValue());
		});
		
		JCheckBox showSecondTimeZoneBox = new JCheckBox();
		showSecondTimeZoneBox.setSelected(s.getSecondTimelineStyle() == Alignment.Near);
		showSecondTimeZoneBox.addActionListener(x ->
		{
			s.setSecondTimelineStyle((showSecondTimeZoneBox.isSelected() ? Alignment.Near : Alignment.None));
		});
		
		JCheckBox showCurrentTimeBox = new JCheckBox();
		showCurrentTimeBox.setSelected(s.getShowCurrentTime());
		showCurrentTimeBox.addActionListener(x ->
		{
			s.setShowCurrentTime(showCurrentTimeBox.isSelected());
		});
		
		JCheckBox timetableNavigationButtonsBox = new JCheckBox();
		timetableNavigationButtonsBox.setSelected(s.getShowNavigationButtons());
		timetableNavigationButtonsBox.addActionListener(x ->
		{
			s.setShowNavigationButtons(timetableNavigationButtonsBox.isSelected());
		});
		
		JSlider minutesPerCellSlider = new JSlider();
		minutesPerCellSlider.setValue(s.getCellTime().getMinutes()); 
		minutesPerCellSlider.addChangeListener(x ->
		{
			Duration value = Duration.fromMinutes(minutesPerCellSlider.getValue());
			Duration min = Duration.fromSeconds(1);
			Duration max = Duration.fromDays(1);
			
			if (Duration.op_LessThanOrEqual(value, max) && Duration.op_GreaterThanOrEqual(value, min))
			{
				s.setCellTime(Duration.fromMinutes(minutesPerCellSlider.getValue()));
			}
		});
		
		JSlider segmentSizeSlider = new JSlider(); 
		segmentSizeSlider.setValue(s.getCellSize());
		segmentSizeSlider.addChangeListener(x -> 
		{
			s.setCellSize(segmentSizeSlider.getValue() >= 10 ? segmentSizeSlider.getValue() : 10);
		});
		
		JButton btnSetDateRange = new JButton("Set Date Range");
		JFrame dataRange = new CalendarView(); 
		btnSetDateRange.addActionListener(x -> 
		{
			 if (dataRange.isActive())
			 {
				 dataRange.dispose();
			 }
			 dataRange.setVisible(true); 
		});
		
		//Put the components in the panel
		JPanel pnlTimeTableSettings = new JPanel();
		
		pnlTimeTableSettings.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = INSETS;
		
		JPanel pnlFirstColumn = (makePanelColumn(
				makePanelRow(new JLabel("Header size : "), timetableHeaderSizeSlider),
				makePanelRow(new JLabel("Sub-header size : "), timetableSubHeaderSizeSlider),
				makePanelRow(timetableHeaderShadowBox, new JLabel("Header shadow"))
				));
		pnlTimeTableSettings.add(pnlFirstColumn, c);
		
		c.gridx = 1;
		JPanel pnlSecondColumn = (makePanelColumn(
				makePanelRow(new JLabel("Timeline size : "), timelineSizeSlider),
				makePanelRow(showSecondTimeZoneBox, new JLabel("Show seconds timeline")),
				makePanelRow(showCurrentTimeBox, new JLabel("Show current time"))
				));
		pnlTimeTableSettings.add(pnlSecondColumn, c);
		
		c.gridx = 2;
		JPanel pnlThirdColumn = (makePanelColumn(
				makePanelRow(new JLabel("Minutes per cell : "), minutesPerCellSlider),
				makePanelRow(new JLabel("Segment size : "), segmentSizeSlider),
				makePanelRow(timetableNavigationButtonsBox, new JLabel("Show navigation buttons"))
				));
		pnlTimeTableSettings.add(pnlThirdColumn, c);
		
		c.gridx = 3;
		JPanel pnlFourthColumn = (makePanelColumn(
				btnSetDateRange
				));
		pnlTimeTableSettings.add(pnlFourthColumn, c);
		
		return pnlTimeTableSettings;
	}

	private JPanel resourceViewSettings(Calendar calendar) 
	{
		JPanel pnlResourceViewSettings = new JPanel();
		
		ResourceViewSettings s = calendar.getResourceViewSettings();
		
		JCheckBox showLanesBox = new JCheckBox();
		showLanesBox.setSelected(s.getViewStyle() == ResourceViewStyle.Lanes);
		showLanesBox.addActionListener(x ->
		{
			s.setViewStyle(showLanesBox.isSelected() ? ResourceViewStyle.Lanes : ResourceViewStyle.General);
		});
		
		JCheckBox groupHeadersBox = new JCheckBox();
		groupHeadersBox.setSelected(s.getGroupRowHeader());
		groupHeadersBox.addActionListener(x -> 
		{
			s.setGroupRowHeader(groupHeadersBox.isSelected());
		});
		
		JCheckBox expandableRowsBox = new JCheckBox();
		expandableRowsBox.setSelected(s.getExpandableRows());
		expandableRowsBox.addActionListener(x -> 
		{
			s.setExpandableRows(expandableRowsBox.isSelected());
		});
		
		//Put the components in the panel
		pnlResourceViewSettings.add(makePanelColumn(
				makePanelRow(showLanesBox, new JLabel("Show lanes")),
				makePanelRow(groupHeadersBox, new JLabel("Group headers")),
				makePanelRow(expandableRowsBox, new JLabel("Expandable rows"))
				));
		
		return pnlResourceViewSettings;
	}
	
	private JPanel singleMonthSettings(Calendar calendar) 
	{	
		MonthSettings s = calendar.getMonthSettings();
		s.setSubHeaderHeight(18); // min value 17 (for visible weekday labels)
	
		JCheckBox monthNavigationButtonsBox = new JCheckBox();
		boolean hasMonthButtons = s.getHeaderStyle().contains(MonthHeaderStyles.MonthButtons);
		monthNavigationButtonsBox.setSelected(hasMonthButtons);
		monthNavigationButtonsBox.addActionListener(x -> 
		{
			EnumSet<MonthHeaderStyles> components = EnumSet.copyOf(s.getHeaderStyle());
			
			if (monthNavigationButtonsBox.isSelected())
			{
				components.add(MonthHeaderStyles.MonthButtons);
				s.setHeaderStyle(components);
			}
			else 
			{
				components.remove(MonthHeaderStyles.MonthButtons);
				s.setHeaderStyle(components);
			}
		});
		
		JCheckBox yearNavigationButtonsBox = new JCheckBox();
		boolean hasYearButtons = s.getHeaderStyle().contains(MonthHeaderStyles.YearButtons);
		yearNavigationButtonsBox.setSelected(hasYearButtons);
		yearNavigationButtonsBox.addActionListener(x -> 
		{
			EnumSet<MonthHeaderStyles> components = EnumSet.copyOf(s.getHeaderStyle());
			
			if (yearNavigationButtonsBox.isSelected())
			{
				components.add(MonthHeaderStyles.YearButtons);
				s.setHeaderStyle(components);
			}
			else 
			{
				components.remove(MonthHeaderStyles.YearButtons);
				s.setHeaderStyle(components);
			}
		});
		
		JCheckBox dayOfWeekBox = new JCheckBox();
		dayOfWeekBox.setSelected(s.getShowDayOfWeek() == true);
		dayOfWeekBox.addActionListener(x -> 
		{
			s.setShowDayOfWeek(dayOfWeekBox.isSelected() ? true : false);
		});
		
		JCheckBox weekNumbersBox = new JCheckBox();
		weekNumbersBox.setSelected(s.getWeekHeaderStyle() != MonthWeekHeaderStyle.None);
		weekNumbersBox.addActionListener(x -> 
		{
			s.setWeekHeaderStyle(weekNumbersBox.isSelected() ? MonthWeekHeaderStyle.Left : MonthWeekHeaderStyle.None);
		});
		
		JCheckBox monthShadowBox = new JCheckBox();
		monthShadowBox.setSelected(s.getHeaderShadowStyle() != ShadowStyle.None);
		monthShadowBox.addActionListener(x ->
		{
			s.setHeaderShadowStyle(monthShadowBox.isSelected() ? ShadowStyle.Fading : ShadowStyle.None);
		});
		
		JSlider monthMainSlider = new JSlider();
		monthMainSlider.setValue(s.getMainHeaderHeight());
		monthMainSlider.addChangeListener(x -> 
		{
			s.setMainHeaderHeight(monthMainSlider.getValue());
		});
		
		JSlider monthSubSlider = new JSlider();
		monthSubSlider.setValue(s.getSubHeaderHeight());
		monthSubSlider.addChangeListener(x -> 
		{
			s.setSubHeaderHeight(monthSubSlider.getValue());
		});
		
		JSlider monthWeekSlider = new JSlider();
		monthWeekSlider.setValue(s.getWeekNumbersHeaderWidth());
		monthWeekSlider.addChangeListener(x -> 
		{
			s.setWeekNumbersHeaderWidth(monthWeekSlider.getValue());
		});
		
		//Put the components in the panel
		JPanel pnlSingleMonthSettings = new JPanel();
		
		pnlSingleMonthSettings.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = INSETS;
		
		JPanel pnlFirstColumn = makePanelColumn(
				makePanelRow(monthNavigationButtonsBox, new JLabel("Month navigation buttons")),
				makePanelRow(yearNavigationButtonsBox, new JLabel("Year navigation buttons")));
		pnlSingleMonthSettings.add(pnlFirstColumn, c);
		
		c.gridx = 1;
		JPanel pnlSecondColumn = makePanelColumn(
				makePanelRow(dayOfWeekBox, new JLabel("Day of week header")),
				makePanelRow(weekNumbersBox, new JLabel("Week numbers header")),
				makePanelRow(monthShadowBox, new JLabel("Header shadow")));
		pnlSingleMonthSettings.add(pnlSecondColumn, c);
		
		c.gridx = 2;
		JPanel pnlThirdColumn = makePanelColumn(
				makePanelRow(new JLabel("Main header : "), monthMainSlider),
				makePanelRow(new JLabel("Sub header : "), monthSubSlider),
				makePanelRow(new JLabel("Week header : "), monthWeekSlider));
		pnlSingleMonthSettings.add(pnlThirdColumn, c);
		
		return pnlSingleMonthSettings;
	}
	
	private JPanel singleMonthCustomizingSettings(Calendar calendar) 
	{
		EnumSet<CustomDrawElements> values = EnumSet.of(CustomDrawElements.MonthMainHeader, CustomDrawElements.MonthSubHeader, 
				CustomDrawElements.MonthSubHeaderCell, CustomDrawElements.MonthWeekNumbersHeader,
				CustomDrawElements.MonthWeekNumbersHeaderCell);
		calendar.setCustomDraw(values);
		
		JButton btnChangeButtonStyle = new JButton("Change button style"); // to-do
		btnChangeButtonStyle.addActionListener(x -> 
		{
			/*
			if (calendar.getControlAppearance() == ControlAppearance.System)
			{
				String message = "You cannot change the appearance of the system controls. " +
						"To toggle to the custom controls, change the theme to Windows2003, Lila, Silver, Vista, or Light.";
				
				JOptionPane.showInternalMessageDialog(calendar, message, "Change Button Style", JOptionPane.OK_OPTION);
				return;
			}
			*/

			ControlStyle style = calendar.getControlStyle();

			calendar.beginInit();

			GradientBrush fillBrush =
				new GradientBrush(Colors.Empty, Colors.Empty, 0);//change

			fillBrush.setAngle(90);
			//fillBrush.setInterpolationColors(new ColorBlend(3));
			fillBrush.setFractions(new float[] { 0, 0.5f, 1 });
			fillBrush.setColors(                               
				new Color[] { Colors.White, Colors.Goldenrod, Colors.Gold });

			GradientBrush overBrush =
				new GradientBrush(Colors.Empty, Colors.Empty, 0);// ? 90

			overBrush.setAngle(90);
			//overBrush.InterpolationColors = new ColorBlend(3);
			overBrush.setFractions(new float[] { 0, 0.5f, 1 });
			overBrush.setColors(
				new Color[] { Colors.White, Colors.Gold, Colors.Yellow });

			GradientBrush downBrush =
				new GradientBrush(Colors.Empty, Colors.Empty, 0);// ? 90

			downBrush.setAngle(90);
			//downBrush.InterpolationColors = new ColorBlend(3);
			downBrush.setFractions(new float[] { 0, 0.5f, 1 });
			downBrush.setColors(new Color[] { Colors.Goldenrod, Colors.DarkGoldenrod, Colors.Gold });

			// Apply the settings
			style.setButtonBorderColor(Colors.DarkGoldenrod);
			style.setButtonFillBrush(fillBrush);
			style.setButtonFillBrushOver(overBrush);
			style.setButtonFillBrushDown(downBrush);

			calendar.endInit();
		});
		
		JButton btnAddHeaderEmbelishments = new JButton("Add header embelishments");
		btnAddHeaderEmbelishments.addActionListener(x -> 
		{
			embelishMonthHeader = true;
			calendar.invalidate();
		});
		
		JButton btnAddFormatingToSomeDates = new JButton("add formating to some dates");
		btnAddFormatingToSomeDates.addActionListener(x -> 
		{	
			if (style1 == null)
			{
				style1 = new DateStyle();
				style1.getStyle().setBorderTopWidth(1);
				style1.getStyle().setBorderLeftWidth(1);
				style1.getStyle().setBorderBottomWidth(1);
				style1.getStyle().setBorderRightWidth(1);
				style1.getStyle().setBrush(new HatchBrush(HatchStyle.SmallCheckerBoard, Colors.PowderBlue, Colors.LightSteelBlue));
				style1.setFrom(new DateTime(calendar.getDate().getYear(), calendar.getDate().getMonth(), 10));
				style1.setTo(new DateTime(calendar.getDate().getYear(), calendar.getDate().getMonth(), 14));

				style2 = new DateStyle(); // doesn't work
				style2.getStyle().setHeaderFont(new Font("Times New Roman", Font.ITALIC, 24));
				style2.getStyle().setHeaderTextColor(Color.RED);
				style2.setFrom(new DateTime(calendar.getDate().getYear(), calendar.getDate().getMonth(), 21));
				style2.setTo(new DateTime(calendar.getDate().getYear(), calendar.getDate().getMonth(), 23));

				style3 = new DateStyle();
				style3.getStyle().setImage(Application.getImages().get(1)); //to-do change number
				style3.getStyle().setImageAlignment(ImageAlignment.Left);
				style3.setFrom(new DateTime(calendar.getDate().getYear(), calendar.getDate().getMonth(), 19));
				style3.setTo(new DateTime(calendar.getDate().getYear(), calendar.getDate().getMonth(), 20));

				style4 = new DateStyle();
				style4.getStyle().setBorderTopWidth(2);
				style4.getStyle().setBorderLeftWidth(2);
				style4.getStyle().setBorderBottomWidth(2);
				style4.getStyle().setBorderRightWidth(2);
				style4.getStyle().setBorderTopColor(Colors.Gainsboro);
				style4.getStyle().setBorderLeftColor(Colors.Gainsboro);
				style4.getStyle().setBorderBottomColor(Colors.LightSlateGray);
				style4.getStyle().setBorderRightColor(Colors.LightSlateGray);
				style4.setFrom(new DateTime(calendar.getDate().getYear(), calendar.getDate().getMonth(), 1));
				style4.setTo(new DateTime(calendar.getDate().getYear(), calendar.getDate().getMonth(), 4));

				calendar.getDayStyles().add(style1);
				calendar.getDayStyles().add(style2);
				calendar.getDayStyles().add(style3);
				calendar.getDayStyles().add(style4);

				calendar.invalidate();
			}
		});
		
		JButton btnClearAll = new JButton("Clear all");
		btnClearAll.addActionListener(x ->
		{
			calendar.beginInit();

			// Reset settings
			ControlStyle style = calendar.getControlStyle();

			style.setButtonBorderColor(Colors.Empty);
			style.setButtonFillBrush(null);
			style.setButtonFillBrushOver(null);
			style.setButtonFillBrushDown(null);

			// Clear formattings
			if (style1 != null)
			{
				calendar.getDayStyles().remove(style1);
				calendar.getDayStyles().remove(style2);
				calendar.getDayStyles().remove(style3);
				calendar.getDayStyles().remove(style4);

				style1 = null;
				style2 = null;
				style3 = null;
				style4 = null;
			}

			embelishMonthHeader = false;

			//calendar.setGroupType(GroupType.None);
			//calendar.getContacts().clear();
			//calendar.getLocations().clear();
			//calendar.getTasks().clear();
			//calendar.getSchedule().clear();

			calendar.endInit();
		});
		
		//Put the components in the panel
		JPanel pnlSingleMonthCustomizingSettings = new JPanel();
		
		pnlSingleMonthCustomizingSettings.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.NORTHWEST;
		c.insets = INSETS;
		
		pnlSingleMonthCustomizingSettings.add(btnChangeButtonStyle, c);
		
		c.gridx = 2;
		pnlSingleMonthCustomizingSettings.add(btnAddHeaderEmbelishments, c);
		
		c.gridx = 3;
		pnlSingleMonthCustomizingSettings.add(btnAddFormatingToSomeDates, c);
		
		c.gridx = 4;
		pnlSingleMonthCustomizingSettings.add(btnClearAll, c);
	
		
		return pnlSingleMonthCustomizingSettings;
	}

	private JPanel makePanelColumn(JComponent... args)
	{
		JPanel pnlToReturn = new JPanel();
		pnlToReturn.setLayout(new GridLayout(args.length, 1));
		
		for (JComponent component : args)
		{
			pnlToReturn.add(component);
		}
		
		return pnlToReturn;
	}
	
	private JPanel makePanelRow(JComponent... args)
	{
		JPanel pnlToReturn = new JPanel();
		pnlToReturn.setLayout(new BoxLayout(pnlToReturn, BoxLayout.X_AXIS));
		
		for (JComponent component : args)
		{
			pnlToReturn.add(component);
		}
		
		return pnlToReturn;
	}
	
	private void addListener(Calendar calendar)
	{
		if (listener == null)
		listener = new CalendarAdapter() 
		{
			@Override
			public void draw(CalendarDrawEvent e) 
			{
				int left = e.getBounds().x;
				int right = e.getBounds().x + e.getBounds().width;
				int bottom = e.getBounds().y + e.getBounds().height;
				int top = e.getBounds().y;
				
				if (currentItem.equals(TreeItems.SINGLE_MONTH_CUSTOMIZING))
				{
					Pen penDark = new Pen(Colors.LightSlateGray);
					Pen penLight = new Pen(Colors.White);
				
					switch (e.getElement())
					{
						case MonthWeekNumbersHeaderCell:
						{
							if (embelishMonthHeader)
							{
								if (e.getIndex() != 5)
								{
									System.out.println("draw line1 + " + e.getBounds());
									e.getGraphics().setColor(penDark.getColor());
									e.getGraphics().drawLine(left + 2, bottom - 1,
															 right - 4, bottom - 1);
								}
								if (e.getIndex() != 0)
								{
									System.out.println("draw line2 + " + e.getBounds());
									e.getGraphics().setColor(penLight.getColor());
									e.getGraphics().drawLine(left + 3, top, 
															 right - 3, top);
								}
							}
						}
						break;

						case MonthSubHeaderCell:
						{
							if (embelishMonthHeader)
							{
								if (e.getIndex() != 6)
								{
									e.getGraphics().setColor(penDark.getColor());
									e.getGraphics().drawLine(right - 1, top + 2,
															 right - 1, bottom - 4);
								}
								if (e.getIndex() != 0)
								{
									e.getGraphics().setColor(penLight.getColor());
									e.getGraphics().drawLine(left, top + 3,
															 left, bottom - 3);
								}
							}
						}
						break;
					}
				}
				
				if (currentItem.equals(TreeItems.TIMETABLE_CUSTOM_DRAWING)) 
				{
					switch (e.getElement())
					{
						case TimetableColumnHeader:
						{
							//if (e.getIndex() < 50)
							{
								Rectangle bounds = e.getBounds();
								int index = (e.getIndex() + 1) % Application.getImages().size();
								BufferedImage image = (BufferedImage) Application.getImages().get(index);

								e.getGraphics().drawImage(image, 
								bounds.x + 2, bounds.y + bounds.height / 2 - image.getHeight() / 2, null);
							}
						}
						break;

						case TimetableCell://to-do
						{
							if (e.getDate().equals(DateTime.today()))
							{
								Duration d1 = new Duration(3, 20, 0);
								Duration d2 = new Duration(5, 0, 0);
								Duration d3 = new Duration(6, 40, 0);
								Duration d4 = new Duration(9, 0, 0);
								
								if ((Duration.op_GreaterThanOrEqual(e.getStartTime(), d1) && Duration.op_LessThan(e.getEndTime(), d2)) ||
									(Duration.op_GreaterThanOrEqual(e.getStartTime(), d3) && Duration.op_LessThan(e.getEndTime(), d4)))
								{
									Rectangle bounds = e.getBounds();
									bounds.width -= 1;
									bounds.height -= 1;

									Brush brush = new SolidBrush(new Color(255, 245, 210));
									brush.applyTo(e.getGraphics(), bounds);
									e.getGraphics().fill(bounds);

									bounds.width -= 1;
									bounds.height -= 1;

									Pen pen = new Pen(Colors.Goldenrod);
									pen.applyTo(e.getGraphics());
									e.getGraphics().draw(bounds);

									BufferedImage image = (BufferedImage) Application.getImages().get(13);

									e.getGraphics().drawImage(image, bounds.x + 2,
										bounds.y + bounds.height / 2 - image.getHeight() / 2, null);
								}
							}
						}
						break;

						case TimetableWholeDayHeader:
						{
							DateTime d = DateTime.today().addMilliseconds(Duration.TicksPerDay);
							if (e.getDate().equals(d)) 
							{
								Brush brush = new HatchBrush(
									HatchStyle.Weave, Colors.SteelBlue, Colors.LightSteelBlue);
								brush.applyTo(e.getGraphics(), e.getBounds());
								e.getGraphics().fill(e.getBounds());

								Pen pen = new Pen(Colors.SteelBlue);
								Rectangle bounds = e.getBounds();
								bounds.width -= 1; 
								bounds.height -= 1; 
								pen.applyTo(e.getGraphics());
								e.getGraphics().draw(bounds);
							}
						}
						break;

					}
				}
				
				if (currentItem.equals(TreeItems.TIMETABLE_CUSTOM_DRAWING_COLORING_CELLS))
				{
					switch (e.getElement())
					{
						case TimetableCell:
						{
							// Draw all workig cells in a custom fashion
							if (e.getDate().getDayOfWeek() != DayOfWeek.Saturday &&
								e.getDate().getDayOfWeek() != DayOfWeek.Sunday)
							{
								// Check if this cell is selected
								DateTime date = e.getDate().add(e.getStartTime());
								boolean selected = calendar.getSelection().contains(date);

								Duration end = Duration.fromHours(calendar.getTimetableSettings().getWorkTimeEndHour());
								Duration start = Duration.fromHours(calendar.getTimetableSettings().getWorkTimeStartHour());
								
								if (Duration.op_LessThan(e.getStartTime(), end) &&
									Duration.op_GreaterThan(e.getEndTime(), start))
								{
									Brush brush = null;
									if (selected)
									{
										brush = new HatchBrush(HatchStyle.Percent50,
											Colors.Goldenrod, Colors.Yellow);
									}
									else
									{
										brush = new HatchBrush(HatchStyle.Percent50,
											Colors.Yellow, Colors.White);
									}

									Rectangle bounds = e.getBounds();
									bounds.width -= 1;
									bounds.height -= 1;

									brush.applyTo(e.getGraphics(), bounds);
									e.getGraphics().fill(bounds);
								}
							}
						}
							break;

					}
				}
				
				if (currentItem.equals(TreeItems.TIMETABLE_CUSTOM_DRAWING_EMBELISHING))
				{
					switch (e.getElement())
					{

						case TimetableTimelineCell:
						{
							if ((embelishTimeline & 1) > 0)
							{
								Pen penDark = new Pen(Colors.LightSlateGray);
								Pen penLight = new Pen(Colors.White);

								Rectangle bounds = e.getBounds();

								// Take into account orientation and
								// right-to-left alignment
								if (calendar.getTimetableSettings().getOrientation().equals(Orientation.Vertical))
								{
									if (calendar.getRightToLeft())
									{
										System.out.println("old w : " + bounds.width);
										bounds.width -= bounds.width / 2;
										System.out.println("new w : " + bounds.width);

										e.getGraphics().setColor(penLight.getColor());
										e.getGraphics().drawLine(
											left + 3, top,
											right, top);
										
										e.getGraphics().setColor(penDark.getColor());
										e.getGraphics().drawLine(
											left + 4, bottom - 1,
											right, bottom - 1);
									}
									else
									{
										bounds.x += bounds.width / 2;
										bounds.width -= bounds.width / 2;

										e.getGraphics().setColor(penLight.getColor());
										e.getGraphics().drawLine(
											left, top,
											right - 3, top);
										
										e.getGraphics().setColor(penDark.getColor());
										e.getGraphics().drawLine(
											left, bottom - 1,
											right - 4, bottom - 1);
									}
								}
								else
								{
									bounds.y += bounds.height / 2;
									bounds.height -= bounds.height / 2;

									if (calendar.getRightToLeft())
										bounds.x += 1;

									e.getGraphics().setColor(penLight.getColor());
									e.getGraphics().drawLine(
										left, top,
										left, bottom - 3);
									
									e.getGraphics().setColor(penDark.getColor());
									e.getGraphics().drawLine(
										right - 1, top,
										right - 1, bottom - 4);
								}
							}
						}
							break;

						case TimetableTimelineHourCell:
						{
							if ((embelishTimeline & 1) > 0)
							{
								Pen penDark = new Pen(Colors.LightSlateGray);
								Pen penLight = new Pen(Colors.White);

								Rectangle bounds = e.getBounds();

								if (calendar.getTimetableSettings().getOrientation().equals(Orientation.Vertical))
								{
									e.getGraphics().setColor(penLight.getColor());
									e.getGraphics().drawLine(
										left + 3, top,
										right - 3, top);
									
									e.getGraphics().setColor(penDark.getColor());
									e.getGraphics().drawLine(
										left + 2, bottom - 1,
										right - 4, bottom - 1);
								}
								else
								{
									if (calendar.getRightToLeft())
										bounds.x += 1;

									e.getGraphics().setColor(penLight.getColor());
									e.getGraphics().drawLine(
										left, top + 3,
										left, bottom - 3);
									
									e.getGraphics().setColor(penDark.getColor());
									e.getGraphics().drawLine(
										right - 1, top + 2,
										right - 1, bottom - 4);
								}
							}

							if ((embelishTimeline & 2) > 1)
							{
								Pen penDark = new Pen(Colors.LightSlateGray);
								Pen penLight = new Pen(Colors.White);

								Rectangle bounds = e.getBounds();
								bounds.width -= 1;
								bounds.height -= 1;

								if (calendar.getRightToLeft())
									bounds.x += 1;

								e.getGraphics().setColor(penLight.getColor());
								e.getGraphics().drawLine(
									left, top,
									right, top);
								e.getGraphics().drawLine(
									left, top,
									left, bottom);
								e.getGraphics().setColor(penDark.getColor());
								e.getGraphics().drawLine(
									right, top,
									right, bottom);
								e.getGraphics().drawLine(
									left, bottom,
									right, bottom);
							}
						}
							break;

					}
				}
			}
		};
		calendar.addCalendarListener(listener);
	}
	
	void removeListener(Calendar calendar)
	{
		calendar.removeCalendarListener(listener);
	}
}
