/**
 * Copyright (c) 2022, MindFusion LLC - Bulgaria.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.Locale;

import javax.swing.*;

import com.mindfusion.scheduling.*;
import com.mindfusion.scheduling.model.*;
import com.mindfusion.scheduling.standardforms.AppointmentForm;
import com.mindfusion.scheduling.standardforms.LocalizationInfo;


public class MainWindow extends JFrame
{
	LocalizationInfo localizationInfo;

	public static void main(String[] args)
	{
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MainWindow window = null;
				try {
					window = new MainWindow();
					window.setVisible(true);
				}
				catch (Exception exp) {
				}
			}
		});
	}

	public MainWindow()
	{
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setSize(600, 600);
		setTitle("MindFusion.Scheduling Sample: Localization");

		calendar = new Calendar();
		calendar.setTheme(ThemeType.Light);

		Container cp = getContentPane();
		cp.setLayout(new BorderLayout());
        cp.add(calendar, BorderLayout.CENTER);
        
        JTextPane label = new JTextPane();
		label.setBackground(new Color(255, 255, 225));
		label.setEditable(false);
		label.setText("This sample demonstrates how to localize the built-in forms. " + 
			"Create an appointment by selecting date range and typing; " + 
			"double click it to open localized form.");
		cp.add(label, BorderLayout.NORTH);

        calendar.setLocale(Locale.forLanguageTag("de-DE"));
        
        localizationInfo = new LocalizationInfo();
		localizationInfo.loadFromXml(
			getClass().getResourceAsStream("Localization.DE.xml"));

		calendar.addCalendarListener(new CalendarAdapter()
		{
			@Override
			public void itemClick(ItemMouseEvent e)
			{
				if (e.getClicks() == 2)
				{
					// Cancel the current drag operation
					calendar.resetDrag();

					// Display the appointment editing form
					AppointmentForm form = new AppointmentForm(
						calendar.getSchedule(), calendar.getDateTimeInfo(), localizationInfo);
					form.setAppointment((Appointment)e.getItem());
					form.setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
					form.setVisible(true);
				}
			}
		});
	}
	
	private Calendar calendar;
	private static final long serialVersionUID = 1L;
}
