

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import com.mindfusion.scheduling.model.Contact;
import com.mindfusion.scheduling.model.ResourceList;

public class ContactInfo extends JFrame 
{
	private final Dimension D_PANEL = new Dimension(130, 50);
	private final Insets INSETS = new Insets(10, 10, 10, 10);
	
	private GridBagConstraints c = new GridBagConstraints();
	
	public ContactInfo(Contact contact)
	{
		super("Contact info");
		
		configureFrame();
		
		makeContactInfoForm(contact);
		
		this.pack();
		this.repaint();
		
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation(dim.width/2-this.getSize().width/2, dim.height/2-this.getSize().height/2);
	}
	
	private void configureFrame() 
	{
		this.setVisible(true);
		this.setLayout(new GridBagLayout());
	}

	private void makeContactInfoForm(Contact contact)
	{
		JTextField txtFirstName = new JTextField(contact == null || contact.getFirstName() == null ? "" : contact.getFirstName());
		JTextField txtLastName = new JTextField(contact == null || contact.getLastName() == null ? "" : contact.getLastName());
		JTextArea txtAddress = new JTextArea(contact == null || contact.getAddress() == null ? "" : contact.getAddress());
		JTextField txtMail = new JTextField(contact == null || contact.getEmail() == null ? "" : contact.getEmail());
		JTextField txtPhone = new JTextField(contact == null || contact.getPhone() == null ? "" : contact.getPhone());
		
		JButton btnOK = new JButton("OK");
		btnOK.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0) 
			{
				Contact newContact = new Contact();
				newContact.setFirstName(txtFirstName.getText());
				newContact.setLastName(txtLastName.getText());
				newContact.setAddress(txtAddress.getText());
				newContact.setEmail(txtMail.getText());
				newContact.setPhone(txtPhone.getText());
				
				ResourceList<Contact> contacts = Application.getInstance().getCalendar().getSchedule().getContacts();
				
				if (contact == null)
				{
					contacts.add(newContact);
				}
				else
				{
					int index = contacts.indexOf(contact);
					contacts.remove(index);
					contacts.add(index, newContact);
				}
				
				ApplicationMenu menu = (ApplicationMenu) Application.getInstance().getJMenuBar();
				menu.getContactsForm().update();
				
				dispose();
			}
		});
		
		JButton btnCancel = new JButton("Cancel");
		btnCancel.addActionListener(new ActionListener()
		{
			@Override
			public void actionPerformed(ActionEvent arg0) 
			{
				dispose();
			}
		});
		
		c.insets = INSETS;
		c.fill = GridBagConstraints.HORIZONTAL;
		
		this.getContentPane().add(createPanel("First Name:", txtFirstName), c); // gridx = 0, gridy = 0, gridwidth = 1;
		
		c.gridx = 1;
		this.getContentPane().add(createPanel("Last Name:", txtLastName), c); // gridx = 1, gridy = 0, gridwidth = 1;
		
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 2;
		this.getContentPane().add(createPanel("Address:", txtAddress), c); // gridx = 0, gridy = 1, gridwidth = 2;
		
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 1;
		this.getContentPane().add(createPanel("E-mail:", txtMail), c); // gridx = 0, gridy = 2, gridwidth = 1;
		
		c.gridx = 1;
		this.getContentPane().add(createPanel("Phone:", txtPhone), c); // gridx = 1, gridy = 2, gridwidth = 1;
		
		c.gridx = 0;
		c.gridy = 3;
		this.getContentPane().add(btnOK, c); // gridx = 0, gridy = 3, gridwidth = 1;
		
		c.gridx = 1;
		this.getContentPane().add(btnCancel, c); // gridx = 1, gridy = 3, gridwidth = 1;
	}
	
	public void setContact(Contact contact) 
	{
		if (contact == null)
			return;
		
		this.removeAll();
		
		configureFrame();
		makeContactInfoForm(contact);
		
		this.pack();
		this.repaint();
	}
	
	private JPanel createPanel(String labelText, JComponent field)
	{
		JPanel panel = new JPanel();
		panel.setLayout(new BoxLayout(panel, BoxLayout.Y_AXIS));
	
		Application.setSizeOf(panel, D_PANEL);
		
		if (field.getClass() == JTextArea.class)
		{
			field.setBorder(new JTextField().getBorder());
			
			Dimension dTextArea = new Dimension(D_PANEL.width, (int) (D_PANEL.height * 1.5)); 
			Application.setSizeOf(panel, dTextArea);
		}
		
		JPanel pnlLabel = new JPanel();
		pnlLabel.setLayout(new FlowLayout(FlowLayout.LEFT));
		
		JLabel label = new JLabel(labelText);
		
		pnlLabel.add(label);
		
		panel.add(pnlLabel);
		panel.add(field);
		
		return panel;
	}
	
}
