

import java.awt.*;
import java.awt.event.*;
import java.util.HashMap;
import java.util.Map;

import javax.swing.*;
import javax.swing.border.Border;

import com.mindfusion.scheduling.*;
import com.mindfusion.scheduling.model.*;

public class ContactsForm extends Form
{
	private final Insets CONTACT_INSETS = new Insets(2, 2, 2, 2);
	private final Border SELECTED_BORDER = BorderFactory.createDashedBorder(Color.black, 5, 5); 
	
	private ContactInfo info;
	private Contact lastSelectedContact;
	private Calendar calendar = Application.getInstance().getCalendar();
	
	Map<Contact, JPanel> contactsPanels;
	
	public ContactsForm(String frameText) 
	{
		super(frameText);
		
		super.getMenu().getBtnAdd().addActionListener(x -> 
		{
			openContactInfoForm(null);
		});	
		
		super.getMenu().getBtnRemove().addActionListener(x ->
		{
			ResourceList<Contact> contacts = calendar.getSchedule().getContacts();
			if (lastSelectedContact != null && contacts.contains(lastSelectedContact))
			{
				contacts.remove(lastSelectedContact);
				update();
			}
		});	
		
		contactsPanels = new HashMap<>();
		
		makeForm();
	}
	
	private void openContactInfoForm(Contact contact)
	{
		if(info != null && info.isShowing())
		{
			info.dispose();
		}
		else 
		{
			info = new ContactInfo(contact);
		}
	}
	
	@Override
	protected void makeForm() 
	{
		JPanel content = new JPanel();
		content.setLayout(new FlowLayout(FlowLayout.LEFT));
		content.setBackground(BACKGROUND);
		
		JPanel allContacts = new JPanel();
		allContacts.setBackground(content.getBackground());
		allContacts.setLayout(new GridBagLayout());
		
		GridBagConstraints c = new GridBagConstraints();
		c.anchor = GridBagConstraints.WEST;
		c.insets = CONTACT_INSETS;
		c.gridx = 0;
		
		ResourceList<Contact> contacts = calendar.getSchedule().getContacts();
		
		// add contacts if there haven't any
		if (contacts.isEmpty() && lastSelectedContact == null)
		{
			addSomeContacts();
		}
		
		// put components into map
		for	(int i = 0; i < contacts.size(); i++)
		{
			Contact currentContact = contacts.get(i);
			
			JPanel pnlContact = makePnlContact(currentContact); 
			
			contactsPanels.put(currentContact, pnlContact);
		}
		
		// visualizing components
		for (int row = 0; row < contacts.size(); row++)
		{
			c.gridy = row;
			
			Contact key = contacts.get(row);
			
			allContacts.add(contactsPanels.get(key), c);
		}
		content.add(allContacts);
		
		// put content into scroll pane 
		JScrollPane pane = new JScrollPane(content);
		pane.setLayout(new ScrollPaneLayout());
		pane.getViewport().setBackground(allContacts.getBackground());
		pane.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED);
		
		this.setContentPane(pane);
	}
	
	private void addSomeContacts()
	{
		Contact c = new Contact();
		c.setFirstName("Petur");
		c.setLastName("Georgiev");
		
		calendar.getSchedule().getContacts().add(c);
		
		Contact c1 = new Contact();
		c1.setFirstName("Geogri");
		c1.setLastName("Petrov");
		
		calendar.getSchedule().getContacts().add(c1);
	}
	
	private JPanel makePnlContact(Contact contact)
	{
		JPanel pnlContact = new JPanel();
		
		pnlContact.setLayout(new FlowLayout(FlowLayout.LEFT));
		pnlContact.add(new JLabel(String.format("%s, %s", contact.getFirstName(), contact.getLastName()))); 
		pnlContact.addMouseListener(new MouseAdapter() 
		{
			@Override
			public void mouseClicked(MouseEvent arg0) 
			{	
				lastSelectedContact = contact;
				
				unselectOtherJPanels();
					
				int mouseClicks = arg0.getClickCount();
						
				if (mouseClicks == 1)
				{	
					selected(pnlContact);
				}
				else if (mouseClicks >= 2) // open contactInfo frame
				{
					openContactInfoForm(lastSelectedContact);
				}
			}
		});
		
		if (lastSelectedContact != null && lastSelectedContact.equals(contact))
		{
			selected(pnlContact);
		}
		else
		{
			unselected(pnlContact);
		}
		
		return pnlContact;
	}
	
	private void unselectOtherJPanels()
	{
		ResourceList<Contact> contacts = calendar.getSchedule().getContacts();
		
		for	(int i = 0; i < contacts.size(); i++)
		{
			Contact current = contacts.get(i);
			
			if (lastSelectedContact != null && !lastSelectedContact.equals(current))
			{
				unselected(contactsPanels.get(current));
			}
		}
		
		super.pack();
		super.repaint();
	}
	
	private void selected(JPanel pnl)
	{
		pnl.setBackground(SELECTED_COLOR);
		pnl.setBorder(SELECTED_BORDER);
	}
	
	private void unselected(JPanel pnl)
	{
		pnl.setBackground(BACKGROUND);
		pnl.setBorder(null);
	}

	@Override
	protected void update() 
	{
		super.getContentPane().removeAll();
		
		makeForm();
		
		super.pack();
		super.repaint();
	}
	
}