

import java.awt.Color;
import java.awt.Desktop;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;

import javax.swing.*;
import javax.swing.border.LineBorder;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkEvent.EventType;
import javax.swing.event.HyperlinkListener;

public class Description extends JPanel  
{
	private static HashMap<TreeItems, String> text = generateDescriptionText(); // to-do : change name 
	
	public Description(TreeItems item)
	{
		this.setLayout(new BoxLayout(this, BoxLayout.X_AXIS));
		
		JTextPane pane = new JTextPane();
		pane.setBackground(new Color(255, 255, 225));
		pane.setBorder(new LineBorder(Color.orange, 1));
		pane.addKeyListener(new KeyboardShortcutsListener()); // can catch shortcut events
		pane.setContentType("text/html");
		pane.setEditable(false);
		pane.addHyperlinkListener(new HyperlinkListener()
		{
			@Override
			public void hyperlinkUpdate(HyperlinkEvent arg)
			{
				
				if (arg.getEventType() == EventType.ACTIVATED)
				{
					try
					{
						URI path = arg.getURL().toURI();
						Desktop.getDesktop().browse(path);
					}
					catch (IOException | URISyntaxException e)
					{	
						e.printStackTrace();
					}
				}
			}	
		});
		
		if (item == null)
			pane.setText(text.get(Application.DEFAULT_TREE_ITEM)); 
		else 
			pane.setText(text.get(item));
		
		pane.setCaretPosition(0);
		
		JScrollPane textAreaScrollPane = new JScrollPane(pane);
	
		textAreaScrollPane.setVerticalScrollBarPolicy(
	                JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		
		this.add(textAreaScrollPane);
	}
	
	private static HashMap<TreeItems, String> generateDescriptionText() 
	{
		HashMap<TreeItems, String> toReturn = new HashMap<>();
		
		toReturn.put(TreeItems.TIMETABLE, "<html>This step illustrates the <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Timetable_View_0.htm\">Timetable</a> view. This view displays one or more days, divided in arbitrary time intervals from 1 to 60 minutes). The view can display one or two timelines on the left, and a header containing the date of each day at the top. <ul><li> Experiment with the controls at the bottom of the window to see how various elements of the view can be customized.</li></ul></html>");
		toReturn.put(TreeItems.VERTICAL_TIMETABLE, "<html>The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Timetable_View_0.htm\">Timetable</a> view can display cells in horizontal or vertical orientation. In vertical view, the time cells for one date are displayed in a column, and the cells for the same time of day are displayed in a row.</html>");
		toReturn.put(TreeItems.HORIZONTAL_TIMETABLE, "<html>The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Timetable_View_0.htm\">Timetable</a> view can display cells in horizontal or vertical orientation. In horizontal view, the time cells for one date are displayed in a row, and the cells for the same time of day are displayed in a column.</html>");
		toReturn.put(TreeItems.TIMETABLE_CUSTOM_DRAWING, "<html>The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Timetable_View_0.htm\">Timetable</a> supports wide variety of custom drawing options, as demonstrated below. The header of each day contains a custom drawn icon. The all-day header of the day after today is drawn with a hatch brush. Some cells are drawn in yellow with double border and an icon.</html>");
		toReturn.put(TreeItems.TIMETABLE_CUSTOM_DRAWING_COLORING_CELLS, "<html>The work time intervals are custom painted using a hatch brush. The selection of time cells is custom drawn too.</html>");
		toReturn.put(TreeItems.TIMETABLE_CUSTOM_DRAWING_EMBELISHING, "<html>This step demonstrates using custom drawing to add adornments to the timeline of a <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Timetable_View_0.htm\">Timetable</a> view. <ul><li>Click on the first button to see one kind of decorations.</li><li>Click on the second button to see a second kind of decorations.</li></ul></html>");
		
		toReturn.put(TreeItems.RESOURCE_VIEW, "<html>The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Resource_View_0.htm\">Resource</a> view displays the distribution of resources in a period of time. The schedule items for each resource are displayed in a single row; the horizontal position of items represents their time span. You can create new items below by double clicking a time cell.</html>");
		
		toReturn.put(TreeItems.SINGLE_MONTH, "<html>This section contains information and samples about the <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Single_Month_View_0.htm\">SingleMonth</a> view. That view displays a single month of the year at a time. The view can optionally display a header showing the month and year name, a header that shows the days of the week and a header that displays the week numbers. <ul><li> You can experiment with the controls at the bottom of the window to see how different aspects of the view can be modified.</li></ul></html>");
		toReturn.put(TreeItems.SINGLE_MONTH_RESPONDING, "<html>This step illustrates how an application could handle actions, such as mouse clicks, performed on the calendar by users. <ul><li> Left-click the calendar header to show a context menu that lets you choose which month to display. </li><li> Right-click on any date to mark it as a holiday or a birthday.</li></ul></html>");
		toReturn.put(TreeItems.SINGLE_MONTH_CUSTOMIZING, "<html>This step shows how to customize the appearance of various elements of the <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Single_Month_View_0.htm\">SingleMonth</a> view, either through the built-in functionality or via custom drawing. <ul><li> Click the 'Change button style' button at the bottom of the window to change the appearance of the navigation buttons. </li><li> Click the 'Add header embellishments' button to adorn the headers with Office2003-like separators. </li><li> Click the 'Add formatting...' button to create custom formatting for several day ranges in the currently displayed month. </li><li> The last button, 'Clear all', undoes the effect of the previous three.</li></ul></html>");
		
		toReturn.put(TreeItems.LIST_VIEW, "<html>This is the <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/List_View_0.htm\">List</a> view, which replaces and extends the obsolete <b>DayRange</b> view. The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/List_View_0.htm\">List</a> view is used to display an arbitrary long sequence of seconds, minutes, hours, days, weeks, months or years. The view can display a header displaying the date of the corresponding day, formatted as set via a .NET <b>DateTimeFormat</b> object. <ul><li> Use the controls at the bottom of the window to change some appearance options of this view.</li></ul></html>");
		toReturn.put(TreeItems.VERTICAL_LIST_VIEW, "<html>The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/List_View_0.htm\">List</a> view supports two layouts - horizontal and vertical (the vertical is used by default). This step illustrates the vertical layout, which arranges cells vertically and displays the header at the left side. <ul><li> Select a cell and start typing to create a new appointment in the schedule.</li></ul></html>");
		toReturn.put(TreeItems.HORIZONTAL_LIST_VIEW, "<html>The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/List_View_0.htm\">List</a> view supports two layouts - horizontal and vertical (the vertical is used by default). This step illustrates the horizontal layout, which arranges cells horizontally and displays the header at the top. <ul><li> Select a cell and start typing to create a new appointment in the schedule.</li></ul></html>");
		
		toReturn.put(TreeItems.WEEK_RANGE, "<html>Here you can see the <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Week_Range_View_0.htm\">WeekRange</a> view. As its name suggests, this view is used to display a sequence of weeks arranged in 7-day rows. The number of days that can be contained in the view at once is unlimited. <br /> This view is very similar to the Calendar view in MS Outlook. <ul><li> As in the other steps, you can adjust some properties of this view using the controls at the bottom of the window.</li></ul></html>");
		toReturn.put(TreeItems.MULTIPLE_WEEK_RANGE, "<html>The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Week_Range_View_0.htm\">WeekRange</a> view supports two layouts for presenting dates - the single week layout and the multiple week layout. In the multiple week layout there are several weeks visible at the same time, each one displayed in its own row. The multiple week layout allows grouping the weekend days in a single cell.</html>");
		toReturn.put(TreeItems.SINGLE_WEEK_RANGE, "<html>The <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Week_Range_View_0.htm\">WeekRange</a> view supports two layouts for presenting dates - the single week layout and the multiple week layout. In the single week layout, there is only one week shown at a time. The days of the displayed week are arranged in 2 columns and 3 rows and the weekend days are always grouped to accommodate for that.</html>");
		
		toReturn.put(TreeItems.MONTH_RANGE, "<html>Here you can see the <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/Month_Range_View_0.htm\">MonthRange</a> view, which is used to display several consecutive months at once. The view usually contains a header and optionally displays navigation buttons. <ul><li> Use the controls at the bottom of the window to change various appearance settings of this view.</li></ul></html>");

		toReturn.put(TreeItems.MISCELLANEOUS, "<html>This category demonstrates some miscellaneous features that can be applied to all types of views.</html>");
		toReturn.put(TreeItems.MISCELLANEOUS_CUSTOM_ITEM_EDITOR, "<html>You can create custom editors for your schedule items as shown below. A custom editor inherits from <b>Control</b> and implements the MindFusion.Scheduling <a href=\"https://www.mindfusion.eu/onlinehelp/jplanner/T_com_mindfusion_scheduling_ItemEditor.htm\">IItemEditor</a> interface. Select several cells and start typing to see a sample editor.</html>");
		toReturn.put(TreeItems.MISCELLANEOUS_FORMATTED_TEXT, "<html>MindFusion.Scheduling can display formatted text in the description area of items, where the text style is specified with html-like formatting tags. Create a new item below and paste the following text to try that: &lt;b&gt;<b>bold</b>&lt;/b&gt;, &lt;i&gt;<i>italic</i>&lt;/i&gt;, &lt;u&gt;<u>underlined</u>&lt;/u&gt; </html>");
		return toReturn;
	}
}
