

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.LineBorder;

public abstract class Form extends JFrame
{
	protected static Dimension SIZE = new Dimension(400, 500);
	protected static Color BACKGROUND = Color.white;
	
	protected static final Color DEFAULT_COLOR = new Color(238, 238, 238); 
	protected static final Color SELECTED_COLOR = new Color(102, 178, 255); 
	
	public Form(String text) 
	{
		super(text);
		
		configureForm();
	}
	
	protected abstract void makeForm();
	protected abstract void update();
	
	protected void configureForm()
	{
		this.setLayout(new FlowLayout(FlowLayout.LEFT));
		this.getContentPane().setBackground(BACKGROUND);
		this.setVisible(true);
		
		Application.setSizeOf(this, SIZE);
		
		this.setJMenuBar(new FormMenu());
		
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		this.setLocation(dim.width/2-this.getSize().width/2, dim.height/2-this.getSize().height/2);
	}
	
	protected FormMenu getMenu()
	{
		return (FormMenu) this.getJMenuBar();
	}
	
	protected class FormMenu extends JMenuBar
	{
		private final LineBorder SELECTED_BORDER = new LineBorder(Color.black, 1);
		
		private final Dimension D_ADD = new Dimension(40, 30);
		private final Dimension D_REMOVE = new Dimension(60, 30);
		
		private final String BTN_ADD_TEXT = "Add";
		private final String BTN_REMOVE_TEXT = "Remove";
		
		private JButton add;
		private JButton remove;
		
		FormMenu()
		{
			this.setBackground(DEFAULT_COLOR);
			
			add = makeButton(add, BTN_ADD_TEXT, D_ADD);
			remove = makeButton(remove, BTN_REMOVE_TEXT, D_REMOVE);
			
			this.add(add);
			this.add(remove);
		}
		
		protected JButton getBtnAdd()
		{
			return add;
		}
		
		protected JButton getBtnRemove()
		{
			return remove;
		}
		
		private JButton makeButton (JButton button, String text, Dimension d)
		{
			button = new JButton(text);
			
			Application.setSizeOf(button, d);
			deselected(button);
			
			button.addMouseListener(new MouseAdapter() 
			{
				@Override
				public void mouseEntered(MouseEvent arg0) 
				{
					if	(text.equals(BTN_ADD_TEXT))
						selected(add);
					else if (text.equals(BTN_REMOVE_TEXT))
						selected(remove);
				}

				@Override
				public void mouseExited(MouseEvent arg0) 
				{
					if	(text.equals(BTN_ADD_TEXT))
						deselected(add);
					else if (text.equals(BTN_REMOVE_TEXT))
						deselected(remove);
				}
			});
			
			return button;
		}
		
		private void selected(JButton button) 
		{
			button.setBackground(SELECTED_COLOR);
			button.setBorder(SELECTED_BORDER);
		}
		
		private void deselected(JButton button)
		{
			button.setBackground(DEFAULT_COLOR);
			button.setBorder(null);
		}
	}
}
