

import java.awt.Color;
import java.awt.FlowLayout;

import javax.swing.JPanel;
import javax.swing.JTree;
import javax.swing.border.LineBorder;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.TreeSelectionModel;

public class TreeModel extends JPanel 
{
	private JTree tree;
	
	public TreeModel()
	{
		this.setLayout(new FlowLayout(FlowLayout.LEFT));
		this.setBorder(new LineBorder(Color.black, 1));
		this.setBackground(Color.white);
		
		DefaultMutableTreeNode root = new DefaultMutableTreeNode("Samples");
		createNodes(root);
		
		tree = new JTree(root);
		tree.setRootVisible(false);
		tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		tree.addKeyListener(new KeyboardShortcutsListener()); // can catch shortcut events
		tree.addTreeSelectionListener(x -> 
		{
			DefaultMutableTreeNode nodeI = (DefaultMutableTreeNode) tree.getLastSelectedPathComponent();
			
			TreeItems newItem = ((Node) nodeI.getUserObject()).getItem();
			Application.getInstance().updateApplication(newItem);
		});
		
		this.add(tree);
	}
	
	private void createNodes(DefaultMutableTreeNode root)
	{
		DefaultMutableTreeNode subcategory = null;
		
		subcategory = new DefaultMutableTreeNode(new Node("Timetable", TreeItems.TIMETABLE)); 
		subcategory.add(new DefaultMutableTreeNode(new Node("Horizontal timetable", TreeItems.HORIZONTAL_TIMETABLE)));
		subcategory.add(new DefaultMutableTreeNode(new Node("Vertical timetable", TreeItems.VERTICAL_TIMETABLE)));
		DefaultMutableTreeNode subcategoryOfTimetable = new DefaultMutableTreeNode(new Node("Custom drawing", TreeItems.TIMETABLE_CUSTOM_DRAWING));
		subcategoryOfTimetable.add(new DefaultMutableTreeNode(new Node("Coloring cells", TreeItems.TIMETABLE_CUSTOM_DRAWING_COLORING_CELLS)));
		subcategoryOfTimetable.add(new DefaultMutableTreeNode(new Node("Embellishing the timeline", TreeItems.TIMETABLE_CUSTOM_DRAWING_EMBELISHING)));
		subcategory.add(subcategoryOfTimetable);
		root.add(subcategory);
	
		subcategory = new DefaultMutableTreeNode(new Node("Resource", TreeItems.RESOURCE_VIEW));
		root.add(subcategory);
		
		subcategory = new DefaultMutableTreeNode(new Node ("Single month", TreeItems.SINGLE_MONTH));
		subcategory.add(new DefaultMutableTreeNode(new Node ("Responding to user actions", TreeItems.SINGLE_MONTH_RESPONDING)));
		subcategory.add(new DefaultMutableTreeNode(new Node ("Customizing the appearance", TreeItems.SINGLE_MONTH_CUSTOMIZING)));
		root.add(subcategory);
		
		subcategory = new DefaultMutableTreeNode(new Node("List View", TreeItems.LIST_VIEW));
		subcategory.add(new DefaultMutableTreeNode(new Node("Horizontal View", TreeItems.HORIZONTAL_LIST_VIEW)));
		subcategory.add(new DefaultMutableTreeNode(new Node("Vertical View", TreeItems.VERTICAL_LIST_VIEW)));
		root.add(subcategory);
		
		subcategory = new DefaultMutableTreeNode(new Node("Week range", TreeItems.WEEK_RANGE));
		subcategory.add(new DefaultMutableTreeNode(new Node("Multiple week", TreeItems.MULTIPLE_WEEK_RANGE)));
		subcategory.add(new DefaultMutableTreeNode(new Node("Single week", TreeItems.SINGLE_WEEK_RANGE)));
		root.add(subcategory);
		
		subcategory = new DefaultMutableTreeNode(new Node("Month range", TreeItems.MONTH_RANGE));
		root.add(subcategory);
		
		subcategory = new DefaultMutableTreeNode(new Node("Miscellaneous", TreeItems.MISCELLANEOUS));
		subcategory.add(new DefaultMutableTreeNode(new Node("Custom item editor", TreeItems.MISCELLANEOUS_CUSTOM_ITEM_EDITOR)));
		subcategory.add(new DefaultMutableTreeNode(new Node("Formatted text", TreeItems.MISCELLANEOUS_FORMATTED_TEXT)));
		root.add(subcategory);
	}
	
	private class Node
	{
		String label;
		TreeItems item;
		
		Node(String label, TreeItems item)
		{
			this.label = label;
			this.item = item;
		}
		
		public TreeItems getItem()
		{
			return item;
		}
		
		@Override
		public String toString()
		{
			return label;
		}
	}
}

